"""
Unit tests for the Main Window v2.1 module
Tests MainWindowV21 - the main application window
"""

from unittest.mock import MagicMock, patch


# Test MainWindowV21 initialization
class TestMainWindowV21Init:
    """Tests for MainWindowV21 initialization"""

    def test_class_exists(self):
        """Test that MainWindowV21 class exists and can be imported"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        assert MainWindowV21 is not None

    def test_class_inherits_from_qmainwindow(self):
        """Test that MainWindowV21 inherits from QMainWindow"""
        from PySide6.QtWidgets import QMainWindow

        from argus_overview.ui.main_window_v21 import MainWindowV21

        assert issubclass(MainWindowV21, QMainWindow)


# Helper to create a mock window without Qt initialization
def create_mock_window():
    """Create a mock MainWindowV21 without Qt initialization"""
    from argus_overview.ui.main_window_v21 import MainWindowV21

    # Create a MagicMock that uses the real methods from MainWindowV21
    window = MagicMock(spec=MainWindowV21)
    window.logger = MagicMock()

    # Bind the real methods to our mock
    window._toggle_visibility = lambda: MainWindowV21._toggle_visibility(window)
    window._toggle_thumbnails = lambda: MainWindowV21._toggle_thumbnails(window)
    window._get_cycling_group_members = lambda: MainWindowV21._get_cycling_group_members(window)
    window._get_window_id_for_character = lambda char: MainWindowV21._get_window_id_for_character(
        window, char
    )
    window._cycle_window = lambda direction=1: MainWindowV21._cycle_window(window, direction)
    window._cycle_next = lambda: MainWindowV21._cycle_next(window)
    window._cycle_prev = lambda: MainWindowV21._cycle_prev(window)
    window._activate_window = lambda wid: MainWindowV21._activate_window(window, wid)
    window._apply_to_all_windows = lambda action: MainWindowV21._apply_to_all_windows(
        window, action
    )
    window._minimize_all_windows = lambda: MainWindowV21._minimize_all_windows(window)
    window._restore_all_windows = lambda: MainWindowV21._restore_all_windows(window)
    window._activate_character = lambda char: MainWindowV21._activate_character(window, char)
    window._on_profile_selected = lambda name: MainWindowV21._on_profile_selected(window, name)
    window._show_settings = lambda: MainWindowV21._show_settings(window)
    window._reload_config = lambda: MainWindowV21._reload_config(window)
    window._quit_application = lambda: MainWindowV21._quit_application(window)
    window._apply_setting = lambda k, v: MainWindowV21._apply_setting(window, k, v)
    window._on_character_detected = lambda wid, char: MainWindowV21._on_character_detected(
        window, wid, char
    )
    window._on_team_selected = lambda team: MainWindowV21._on_team_selected(window, team)
    window._on_layout_applied = lambda name: MainWindowV21._on_layout_applied(window, name)
    window.closeEvent = lambda e: MainWindowV21.closeEvent(window, e)
    window._show_about_dialog = lambda: MainWindowV21._show_about_dialog(window)
    window._open_url = lambda url: MainWindowV21._open_url(window, url)
    window._open_donation_link = lambda: MainWindowV21._open_donation_link(window)
    window._on_new_character_discovered = lambda c, wid, t: (
        MainWindowV21._on_new_character_discovered(window, c, wid, t)
    )
    window._apply_low_power_mode = lambda enabled: MainWindowV21._apply_low_power_mode(
        window, enabled
    )

    # Provide capture_system mock with _window_mgr for _activate_window
    mock_window_mgr = MagicMock()
    mock_window_mgr.is_valid_window_id.return_value = True
    window.capture_system = MagicMock()
    window.capture_system._window_mgr = mock_window_mgr

    return window


# Test visibility toggle
class TestToggleVisibility:
    """Tests for _toggle_visibility method"""

    def test_toggle_visibility_hides_when_visible(self):
        """Test that toggle hides window when visible"""
        window = create_mock_window()

        # Mock methods
        window.isVisible = MagicMock(return_value=True)
        window.hide = MagicMock()
        window.show = MagicMock()

        window._toggle_visibility()

        window.hide.assert_called_once()
        window.show.assert_not_called()

    def test_toggle_visibility_shows_when_hidden(self):
        """Test that toggle shows window when hidden"""
        window = create_mock_window()

        # Mock methods
        window.isVisible = MagicMock(return_value=False)
        window.hide = MagicMock()
        window.show = MagicMock()
        window.raise_ = MagicMock()
        window.activateWindow = MagicMock()

        window._toggle_visibility()

        window.show.assert_called_once()
        window.raise_.assert_called_once()
        window.activateWindow.assert_called_once()


# Test toggle thumbnails
class TestToggleThumbnails:
    """Tests for _toggle_thumbnails method"""

    def test_toggle_thumbnails_calls_main_tab(self):
        """Test that toggle calls main_tab method"""
        window = create_mock_window()
        window.main_tab = MagicMock()

        window._toggle_thumbnails()

        window.main_tab.toggle_thumbnails_visibility.assert_called_once()

    def test_toggle_thumbnails_handles_no_main_tab(self):
        """Test that toggle handles missing main_tab gracefully"""
        window = create_mock_window()
        # No main_tab attribute

        # Should not raise
        window._toggle_thumbnails()


# Test cycling group members
class TestCyclingGroupMembers:
    """Tests for _get_cycling_group_members method"""

    def test_get_cycling_group_members_returns_current_group(self):
        """Test getting members from current cycling group"""
        window = create_mock_window()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = {
            "Default": ["Char1", "Char2"],
            "PvP": ["Char3", "Char4"],
        }
        window.current_cycling_group = "PvP"

        result = window._get_cycling_group_members()

        assert result == ["Char3", "Char4"]

    def test_get_cycling_group_members_fallback_to_default(self):
        """Test fallback to Default group"""
        window = create_mock_window()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = {"Default": ["Char1", "Char2"]}
        window.current_cycling_group = "NonExistent"

        result = window._get_cycling_group_members()

        assert result == ["Char1", "Char2"]


# Test window ID lookup
class TestGetWindowIdForCharacter:
    """Tests for _get_window_id_for_character method"""

    def test_get_window_id_found(self):
        """Test finding window ID for character"""
        window = create_mock_window()

        # Mock main_tab with window_manager
        mock_frame = MagicMock()
        mock_frame.character_name = "TestPilot"

        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {"0x12345": mock_frame}

        result = window._get_window_id_for_character("TestPilot")

        assert result == "0x12345"

    def test_get_window_id_not_found(self):
        """Test window ID not found returns None"""
        window = create_mock_window()

        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {}

        result = window._get_window_id_for_character("Unknown")

        assert result is None


# Test cycle next/prev
class TestCycling:
    """Tests for _cycle_next and _cycle_prev methods"""

    def test_cycle_next_advances_index(self):
        """Test cycle_next advances cycling index"""
        window = create_mock_window()
        window.cycling_index = 0
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = {"Default": ["Char1", "Char2", "Char3"]}
        window.current_cycling_group = "Default"

        # Mock finding window
        mock_frame = MagicMock()
        mock_frame.character_name = "Char2"
        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {"0x12345": mock_frame}

        window._activate_window = MagicMock()

        window._cycle_next()

        assert window.cycling_index == 1

    def test_cycle_next_wraps_around(self):
        """Test cycle_next wraps to beginning"""
        window = create_mock_window()
        window.cycling_index = 2  # Last position
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = {"Default": ["Char1", "Char2", "Char3"]}
        window.current_cycling_group = "Default"

        # Mock finding window
        mock_frame = MagicMock()
        mock_frame.character_name = "Char1"
        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {"0x12345": mock_frame}

        window._activate_window = MagicMock()

        window._cycle_next()

        assert window.cycling_index == 0  # Wrapped to beginning

    def test_cycle_prev_decrements_index(self):
        """Test cycle_prev decrements cycling index"""
        window = create_mock_window()
        window.cycling_index = 2
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = {"Default": ["Char1", "Char2", "Char3"]}
        window.current_cycling_group = "Default"

        # Mock finding window
        mock_frame = MagicMock()
        mock_frame.character_name = "Char2"
        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {"0x12345": mock_frame}

        window._activate_window = MagicMock()

        window._cycle_prev()

        assert window.cycling_index == 1


# Test activate window (uses create_mock_window with capture_system)
class TestActivateWindowBasic:
    """Tests for _activate_window method (basic)"""

    def test_activate_window_calls_capture_system(self):
        """Test that activate_window delegates to capture_system"""
        window = create_mock_window()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = False  # auto_minimize off

        window._activate_window("0x12345")

        window.capture_system.activate_window.assert_called_once_with("0x12345")

    def test_activate_window_handles_exception(self):
        """Test that activate_window handles exceptions"""
        window = create_mock_window()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = False
        window.capture_system.activate_window.side_effect = OSError("failed")

        # Should not raise
        window._activate_window("0x12345")

        window.logger.error.assert_called()


# Test minimize/restore all windows
class TestMinimizeRestoreWindows:
    """Tests for _minimize_all_windows and _restore_all_windows"""

    def test_minimize_all_windows(self):
        """Test minimizing all EVE windows"""
        window = create_mock_window()

        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {"0x111": MagicMock(), "0x222": MagicMock()}

        window.capture_system = MagicMock()
        window.capture_system.minimize_window.return_value = True

        window.system_tray = MagicMock()

        window._minimize_all_windows()

        assert window.capture_system.minimize_window.call_count == 2
        window.system_tray.show_notification.assert_called()

    def test_restore_all_windows(self):
        """Test restoring all EVE windows"""
        window = create_mock_window()

        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {"0x111": MagicMock(), "0x222": MagicMock()}

        window.capture_system = MagicMock()
        window.capture_system.restore_window.return_value = True

        window.system_tray = MagicMock()

        window._restore_all_windows()

        assert window.capture_system.restore_window.call_count == 2
        window.system_tray.show_notification.assert_called()


# Test activate character
class TestActivateCharacter:
    """Tests for _activate_character method"""

    def test_activate_character_found(self):
        """Test activating a found character"""
        window = create_mock_window()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = False  # auto_minimize off

        mock_frame = MagicMock()
        mock_frame.character_name = "TestPilot"

        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {"0x12345": mock_frame}

        window._activate_character("TestPilot")

        # Should delegate to capture_system.activate_window via _activate_window
        window.capture_system.activate_window.assert_called_once_with("0x12345")

    def test_activate_character_not_found(self):
        """Test activating a character not found"""
        window = create_mock_window()

        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {}

        window._activate_character("Unknown")

        window.logger.warning.assert_called()


# Test profile selection
class TestProfileSelection:
    """Tests for _on_profile_selected method"""

    def test_on_profile_selected_loads_preset(self):
        """Test that profile selection loads preset"""
        window = create_mock_window()

        mock_preset = MagicMock()
        window.layout_manager = MagicMock()
        window.layout_manager.get_preset.return_value = mock_preset

        window.system_tray = MagicMock()

        window._on_profile_selected("MyProfile")

        window.layout_manager.get_preset.assert_called_with("MyProfile")
        window.system_tray.set_current_profile.assert_called_with("MyProfile")


# Test show settings
class TestShowSettings:
    """Tests for _show_settings method"""

    def test_show_settings_switches_to_tab(self):
        """Test that show_settings shows window and switches tab"""
        window = create_mock_window()
        window.show = MagicMock()
        window.raise_ = MagicMock()
        window.tabs = MagicMock()

        window._show_settings()

        window.show.assert_called_once()
        window.raise_.assert_called_once()
        window.tabs.setCurrentIndex.assert_called_with(4)


# Test reload config
class TestReloadConfig:
    """Tests for _reload_config method"""

    def test_reload_config_reloads_settings(self):
        """Test that reload_config reloads all settings"""
        window = create_mock_window()

        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = True

        window.theme_manager = MagicMock()

        window.auto_discovery = MagicMock()
        window.auto_discovery.scan_timer = MagicMock()
        window.auto_discovery.scan_timer.isActive.return_value = False

        window.system_tray = MagicMock()

        window._apply_initial_settings = MagicMock()

        window._reload_config()

        window.settings_manager.load_settings.assert_called_once()
        window._apply_initial_settings.assert_called_once()
        window.theme_manager.apply_theme.assert_called_once()
        window.system_tray.show_notification.assert_called()


# Test quit application
class TestQuitApplication:
    """Tests for _quit_application method"""

    @patch("argus_overview.ui.main_window_v21.QApplication")
    def test_quit_application_calls_quit(self, mock_app):
        """Test that quit_application calls QApplication.quit"""
        window = create_mock_window()

        window._quit_application()

        mock_app.quit.assert_called_once()


# Test apply setting
class TestApplySetting:
    """Tests for _apply_setting method"""

    def test_apply_setting_performance(self):
        """Test applying performance setting"""
        window = create_mock_window()

        window._apply_setting("performance.capture_workers", 8)

        window.logger.info.assert_called()
        window.logger.warning.assert_called()


# Test character detected
class TestOnCharacterDetected:
    """Tests for _on_character_detected slot"""

    def test_on_character_detected_assigns_window(self):
        """Test that character detection assigns window"""
        window = create_mock_window()
        window.character_manager = MagicMock()

        window._on_character_detected("0x12345", "TestPilot")

        window.character_manager.assign_window.assert_called_with("TestPilot", "0x12345")


# Test team selected
class TestOnTeamSelected:
    """Tests for _on_team_selected slot"""

    def test_on_team_selected_logs_team_name(self):
        """Test that team selection logs the team name"""
        window = create_mock_window()

        mock_team = MagicMock()
        mock_team.name = "Fleet1"

        window._on_team_selected(mock_team)

        window.logger.info.assert_called_with("Team selected: Fleet1")


# Test layout applied
class TestOnLayoutApplied:
    """Tests for _on_layout_applied slot"""

    def test_on_layout_applied_logs(self):
        """Test that layout applied logs message"""
        window = create_mock_window()

        window._on_layout_applied("MyLayout")

        window.logger.info.assert_called()


# Test close event
class TestCloseEvent:
    """Tests for closeEvent handler"""

    def test_close_event_minimizes_to_tray(self):
        """Test that close minimizes to tray when enabled"""
        window = create_mock_window()

        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = True  # minimize_to_tray enabled

        window.system_tray = MagicMock()
        window.system_tray.is_visible.return_value = True

        window.hide = MagicMock()

        mock_event = MagicMock()

        window.closeEvent(mock_event)

        window.hide.assert_called_once()
        mock_event.ignore.assert_called_once()

    def test_close_event_actually_closes(self):
        """Test that close actually closes when tray disabled"""
        window = create_mock_window()

        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = False  # minimize_to_tray disabled

        window.auto_discovery = MagicMock()
        window.capture_system = MagicMock()
        window.hotkey_manager = MagicMock()
        window.system_tray = MagicMock()
        window.character_manager = MagicMock()

        mock_event = MagicMock()

        window.closeEvent(mock_event)

        window.auto_discovery.stop.assert_called_once()
        window.capture_system.stop.assert_called_once()
        window.hotkey_manager.stop.assert_called_once()
        window.settings_manager.save_settings.assert_called_once()
        window.character_manager.save_data.assert_called_once()
        mock_event.accept.assert_called_once()


# Test about dialog
class TestAboutDialog:
    """Tests for _show_about_dialog method"""

    @patch("argus_overview.ui.about_dialog.AboutDialog")
    def test_show_about_dialog_creates_dialog(self, mock_dialog_class):
        """Test that show_about_dialog creates and shows dialog"""
        window = create_mock_window()

        mock_dialog = MagicMock()
        mock_dialog_class.return_value = mock_dialog

        window._show_about_dialog()

        mock_dialog_class.assert_called_once_with(window)
        mock_dialog.exec.assert_called_once()


# Test open URL
class TestOpenUrl:
    """Tests for _open_url and _open_donation_link methods"""

    @patch("PySide6.QtGui.QDesktopServices.openUrl")
    def test_open_url(self, mock_open_url):
        """Test opening URL"""
        window = create_mock_window()

        window._open_url("https://example.com")

        mock_open_url.assert_called_once()

    @patch("PySide6.QtGui.QDesktopServices.openUrl")
    def test_open_donation_link(self, mock_open_url):
        """Test opening donation link"""
        window = create_mock_window()

        window._open_donation_link()

        mock_open_url.assert_called_once()


# Test new character discovered
class TestNewCharacterDiscovered:
    """Tests for _on_new_character_discovered slot"""

    def test_on_new_character_discovered_adds_window(self):
        """Test that new character adds window to main tab"""
        window = create_mock_window()

        # Mock main_tab
        mock_frame = MagicMock()
        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {}  # Not already there
        window.main_tab.window_manager.add_window.return_value = mock_frame

        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = True  # show_notifications

        window.system_tray = MagicMock()

        window._on_new_character_discovered("NewPilot", "0x99999", "EVE - NewPilot")

        window.main_tab.window_manager.add_window.assert_called_with("0x99999", "NewPilot")
        window.system_tray.show_notification.assert_called()


# Test toggle lock
class TestToggleLock:
    """Tests for _toggle_lock method"""

    def test_toggle_lock_clicks_lock_button(self):
        """Test that toggle_lock clicks the lock button"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.main_tab = MagicMock()
        window.main_tab.lock_btn = MagicMock()

        MainWindowV21._toggle_lock(window)

        window.main_tab.lock_btn.click.assert_called_once()

    def test_toggle_lock_no_main_tab(self):
        """Test toggle_lock handles missing main_tab gracefully"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        # No main_tab attribute
        del window.main_tab

        # Should not raise
        MainWindowV21._toggle_lock(window)


# Test get cycling group members edge cases
class TestGetCyclingGroupMembersEdgeCases:
    """Edge case tests for _get_cycling_group_members"""

    def test_fallback_to_default_group(self):
        """Test fallback to Default group when current not found"""
        window = create_mock_window()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = {"Default": ["Char1", "Char2"]}
        window.current_cycling_group = "NonExistent"

        members = window._get_cycling_group_members()

        assert members == ["Char1", "Char2"]

    def test_fallback_to_active_windows(self):
        """Test fallback to active windows when no groups defined"""
        window = create_mock_window()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = {}
        window.current_cycling_group = "Default"

        mock_frame1 = MagicMock()
        mock_frame1.character_name = "ActiveChar1"
        mock_frame2 = MagicMock()
        mock_frame2.character_name = "ActiveChar2"

        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {"0x111": mock_frame1, "0x222": mock_frame2}

        members = window._get_cycling_group_members()

        assert "ActiveChar1" in members
        assert "ActiveChar2" in members


# Test cycle when character not found
class TestCycleEdgeCases:
    """Edge case tests for cycling methods"""

    def test_cycle_next_empty_group(self):
        """Test cycle_next with empty group"""
        window = create_mock_window()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = {}
        window.current_cycling_group = "Empty"

        # No main_tab to fall back to
        del window.main_tab

        window._cycle_next()

        window.logger.warning.assert_called()

    def test_cycle_prev_empty_group(self):
        """Test cycle_prev with empty group"""
        window = create_mock_window()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = {}
        window.current_cycling_group = "Empty"

        # No main_tab to fall back to
        del window.main_tab

        window._cycle_prev()

        window.logger.warning.assert_called()


# Test handle hotkey
class TestHandleHotkey:
    """Tests for _handle_hotkey method"""

    def test_handle_hotkey_logs_message(self):
        """Test that _handle_hotkey logs the hotkey name"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()

        MainWindowV21._handle_hotkey(window, "test_hotkey")

        window.logger.info.assert_called()


# Test reload config edge cases
class TestReloadConfigEdgeCases:
    """Edge case tests for _reload_config"""

    def test_reload_config_stops_auto_discovery_when_disabled(self):
        """Test that reload_config stops auto-discovery when disabled"""
        window = create_mock_window()

        window.settings_manager = MagicMock()
        # First call returns theme, second call returns False for auto_discovery
        window.settings_manager.get.side_effect = [
            "dark",  # appearance.theme
            False,  # general.auto_discovery
        ]

        window.theme_manager = MagicMock()
        window.auto_discovery = MagicMock()
        window.system_tray = MagicMock()
        window._apply_initial_settings = MagicMock()

        window._reload_config()

        window.auto_discovery.stop.assert_called_once()

    def test_reload_config_updates_running_auto_discovery(self):
        """Test reload_config updates interval when auto-discovery running"""
        window = create_mock_window()

        window.settings_manager = MagicMock()
        window.settings_manager.get.side_effect = [
            "dark",  # appearance.theme
            True,  # general.auto_discovery
            10,  # general.auto_discovery_interval
        ]

        window.theme_manager = MagicMock()
        window.auto_discovery = MagicMock()
        window.auto_discovery.scan_timer = MagicMock()
        window.auto_discovery.scan_timer.isActive.return_value = True  # Already running

        window.system_tray = MagicMock()
        window._apply_initial_settings = MagicMock()

        window._reload_config()

        window.auto_discovery.set_interval.assert_called_with(10)
        window.auto_discovery.start.assert_not_called()  # Already running


# Test apply setting edge cases
class TestApplySettingEdgeCases:
    """Edge case tests for _apply_setting"""

    def test_apply_setting_hotkeys(self):
        """Test applying hotkeys setting (currently a no-op)"""
        window = create_mock_window()

        # Should not raise
        window._apply_setting("hotkeys.minimize_all", "<ctrl>+m")

        window.logger.info.assert_called()

    def test_apply_setting_performance_refresh_rate(self):
        """Test applying refresh rate setting"""
        window = create_mock_window()

        # Should not raise
        window._apply_setting("performance.default_refresh_rate", 60)

        window.logger.info.assert_called()


# Test on character detected with status update
class TestOnCharacterDetectedEdgeCases:
    """Edge case tests for _on_character_detected"""

    def test_on_character_detected_updates_characters_tab(self):
        """Test that character detection updates characters tab if available"""
        window = create_mock_window()
        window.character_manager = MagicMock()
        window.characters_tab = MagicMock()

        window._on_character_detected("0x12345", "TestPilot")

        window.characters_tab.update_character_status.assert_called_with("TestPilot", "0x12345")


# Test get window id for character
class TestGetWindowIdForCharacter:
    """Tests for _get_window_id_for_character method"""

    def test_get_window_id_not_found(self):
        """Test returns None when character not found"""
        window = create_mock_window()

        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {}

        result = window._get_window_id_for_character("Unknown")

        assert result is None

    def test_get_window_id_no_main_tab(self):
        """Test returns None when no main_tab"""
        window = create_mock_window()
        del window.main_tab

        result = window._get_window_id_for_character("SomeChar")

        assert result is None


# Test apply setting with main_tab for refresh rate
class TestApplySettingRefreshRate:
    """Tests for _apply_setting with performance.default_refresh_rate"""

    def test_apply_setting_refresh_rate_with_main_tab(self):
        """Test applying refresh rate setting when main_tab exists"""
        window = create_mock_window()
        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()

        window._apply_setting("performance.default_refresh_rate", 30)

        window.main_tab.window_manager.set_refresh_rate.assert_called_once_with(30)

    def test_apply_setting_disable_previews_with_main_tab(self):
        """Test applying disable_previews setting when main_tab exists"""
        window = create_mock_window()
        window.main_tab = MagicMock()

        window._apply_setting("performance.disable_previews", True)

        window.main_tab.set_previews_enabled.assert_called_once_with(False)

    def test_apply_setting_disable_previews_false(self):
        """Test applying disable_previews=False"""
        window = create_mock_window()
        window.main_tab = MagicMock()

        window._apply_setting("performance.disable_previews", False)

        window.main_tab.set_previews_enabled.assert_called_once_with(True)


# Test cycling recursion edge case
class TestCyclingRecursion:
    """Tests for cycling recursion when character not found"""

    def test_cycle_next_recursion_on_not_found(self):
        """Test _cycle_next recursively tries next when not found"""
        window = create_mock_window()
        window.cycling_index = 0
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = {"Default": ["NotFound", "FoundChar"]}
        window.current_cycling_group = "Default"

        # First char not found, second char found
        mock_frame = MagicMock()
        mock_frame.character_name = "FoundChar"

        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {
            "0x222": mock_frame  # Only FoundChar exists
        }

        window._activate_window = MagicMock()

        window._cycle_next()

        # Should have advanced to index 1 (FoundChar) after not finding NotFound
        assert window.cycling_index == 1 or window._activate_window.called

    def test_cycle_prev_recursion_on_not_found(self):
        """Test _cycle_prev recursively tries prev when not found"""
        window = create_mock_window()
        window.cycling_index = 1
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = {"Default": ["FoundChar", "NotFound"]}
        window.current_cycling_group = "Default"

        # Second char not found, first char found
        mock_frame = MagicMock()
        mock_frame.character_name = "FoundChar"

        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {
            "0x111": mock_frame  # Only FoundChar exists
        }

        window._activate_window = MagicMock()

        window._cycle_prev()

        # Should have decremented to find FoundChar
        assert window._activate_window.called or window.cycling_index == 0


# Test _set_window_icon
class TestSetWindowIcon:
    """Tests for _set_window_icon method"""

    @patch("argus_overview.ui.main_window_v21.Path")
    def test_set_window_icon_found(self, mock_path_class):
        """Test setting window icon when icon file found"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.setWindowIcon = MagicMock()

        # Mock path that exists
        mock_path = MagicMock()
        mock_path.exists.return_value = True
        mock_path.__str__ = MagicMock(return_value="/path/to/icon.png")
        mock_path_class.return_value = mock_path
        mock_path_class.__truediv__ = lambda s, o: mock_path

        # Mock Path.home() to return a mock that constructs valid paths
        with patch.object(mock_path_class, "home", return_value=mock_path):
            with patch.object(mock_path_class, "__call__", return_value=mock_path):
                MainWindowV21._set_window_icon(window)

        # Should have called setWindowIcon (at least once somewhere)
        # The implementation checks multiple paths

    @patch("argus_overview.ui.main_window_v21.Path")
    def test_set_window_icon_not_found(self, mock_path_class):
        """Test warning when no icon found"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.setWindowIcon = MagicMock()

        # Create a mock path that always returns a path that doesn't exist
        def make_mock_path():
            mock_path = MagicMock()
            mock_path.exists.return_value = False
            mock_path.parent = mock_path  # .parent returns itself
            mock_path.__truediv__ = lambda self, other: make_mock_path()  # / returns new mock
            return mock_path

        mock_path = make_mock_path()
        mock_path_class.return_value = mock_path
        mock_path_class.home.return_value = mock_path

        MainWindowV21._set_window_icon(window)

        window.logger.warning.assert_called()


# Test _apply_initial_settings
class TestApplyInitialSettings:
    """Tests for _apply_initial_settings method"""

    def test_apply_initial_settings(self):
        """Test that _apply_initial_settings applies all settings"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = 4  # Default return for all gets

        window.capture_system = MagicMock()

        MainWindowV21._apply_initial_settings(window)

        # Should update capture_system.max_workers
        assert window.capture_system.max_workers == 4


# Test _connect_signals
class TestConnectSignals:
    """Tests for _connect_signals method"""

    def test_connect_signals(self):
        """Test that _connect_signals logs debug message"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()

        MainWindowV21._connect_signals(window)

        window.logger.debug.assert_called()


# Test profile not found
class TestProfileNotFound:
    """Test profile selection when preset not found"""

    def test_on_profile_selected_not_found(self):
        """Test profile selection when preset doesn't exist"""
        window = create_mock_window()

        window.layout_manager = MagicMock()
        window.layout_manager.get_preset.return_value = None  # Not found

        window.system_tray = MagicMock()

        window._on_profile_selected("NonExistent")

        window.layout_manager.get_preset.assert_called_with("NonExistent")
        # Should not call set_current_profile when preset not found
        window.system_tray.set_current_profile.assert_not_called()


# Test new character discovered - already exists
class TestNewCharacterAlreadyExists:
    """Test _on_new_character_discovered when character already tracked"""

    def test_on_new_character_discovered_already_exists(self):
        """Test that nothing happens when character already exists"""
        window = create_mock_window()

        # Character already in preview_frames
        mock_frame = MagicMock()
        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {
            "0x99999": mock_frame  # Already exists
        }

        window.system_tray = MagicMock()

        window._on_new_character_discovered("ExistingPilot", "0x99999", "EVE - ExistingPilot")

        # add_window should NOT be called
        window.main_tab.window_manager.add_window.assert_not_called()


# Test new character discovered - no notification
class TestNewCharacterNoNotification:
    """Test _on_new_character_discovered without notifications"""

    def test_on_new_character_discovered_no_notification(self):
        """Test that notification is skipped when disabled"""
        window = create_mock_window()

        mock_frame = MagicMock()
        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {}
        window.main_tab.window_manager.add_window.return_value = mock_frame
        window.main_tab.preview_layout = MagicMock()

        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = False  # show_notifications disabled

        window.system_tray = MagicMock()

        window._on_new_character_discovered("NewPilot", "0x88888", "EVE - NewPilot")

        # add_window should be called
        window.main_tab.window_manager.add_window.assert_called_once()
        # But show_notification should NOT be called
        window.system_tray.show_notification.assert_not_called()


# Test new character discovered - frame is None
class TestNewCharacterFrameNone:
    """Test _on_new_character_discovered when add_window returns None"""

    def test_on_new_character_discovered_frame_none(self):
        """Test handling when add_window returns None"""
        window = create_mock_window()

        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {}
        window.main_tab.window_manager.add_window.return_value = None  # Failed to create

        window._on_new_character_discovered("NewPilot", "0x77777", "EVE - NewPilot")

        # Should not try to connect signals on None
        window.main_tab.preview_layout.addWidget.assert_not_called()


# Test minimize/restore handles no main_tab
class TestMinimizeRestoreNoMainTab:
    """Tests for minimize/restore when main_tab missing"""

    def test_minimize_all_no_main_tab(self):
        """Test minimize_all handles missing main_tab gracefully"""
        window = create_mock_window()
        del window.main_tab

        # Should not raise
        window._minimize_all_windows()

    def test_restore_all_no_main_tab(self):
        """Test restore_all handles missing main_tab gracefully"""
        window = create_mock_window()
        del window.main_tab

        # Should not raise
        window._restore_all_windows()


# Test activate character no main_tab
class TestActivateCharacterNoMainTab:
    """Test _activate_character when main_tab missing"""

    def test_activate_character_no_main_tab(self):
        """Test activate_character handles missing main_tab"""
        window = create_mock_window()
        del window.main_tab

        window._activate_character("SomeChar")

        window.logger.warning.assert_called()


# Test _register_hotkeys
class TestRegisterHotkeys:
    """Tests for _register_hotkeys method"""

    def test_register_hotkeys_basic(self):
        """Test registering basic hotkeys"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.settings_manager = MagicMock()
        window.settings_manager.get.side_effect = lambda key, default=None: {
            "hotkeys.minimize_all": "<ctrl>+<shift>+m",
            "hotkeys.restore_all": "<ctrl>+<shift>+r",
            "hotkeys.toggle_thumbnails": "<ctrl>+<shift>+t",
            "hotkeys.toggle_lock": "<ctrl>+<shift>+l",
            "character_hotkeys": {},
            "hotkeys.cycle_next": "<ctrl>+<tab>",
            "hotkeys.cycle_prev": "<ctrl>+<shift>+<tab>",
        }.get(key, default)

        window.hotkey_manager = MagicMock()

        MainWindowV21._register_hotkeys(window)

        # Should register basic hotkeys
        assert window.hotkey_manager.register_hotkey.call_count >= 4
        window.logger.info.assert_called()

    def test_register_hotkeys_with_character_hotkeys(self):
        """Test registering per-character hotkeys"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.settings_manager = MagicMock()
        window.settings_manager.get.side_effect = lambda key, default=None: {
            "hotkeys.minimize_all": "<ctrl>+m",
            "hotkeys.restore_all": "<ctrl>+r",
            "hotkeys.toggle_thumbnails": "<ctrl>+t",
            "hotkeys.toggle_lock": "<ctrl>+l",
            "character_hotkeys": {"Pilot1": "<f1>", "Pilot2": "<f2>"},
            "hotkeys.cycle_next": "<ctrl>+<tab>",
            "hotkeys.cycle_prev": "<ctrl>+<shift>+<tab>",
        }.get(key, default)

        window.hotkey_manager = MagicMock()

        MainWindowV21._register_hotkeys(window)

        # Should register character hotkeys (4 basic + 2 chars + 2 cycling = 8+)
        assert window.hotkey_manager.register_hotkey.call_count >= 6

    def test_register_hotkeys_no_cycle_combos(self):
        """Test registering hotkeys when cycle combos are empty"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.settings_manager = MagicMock()
        window.settings_manager.get.side_effect = lambda key, default=None: {
            "hotkeys.minimize_all": "<ctrl>+m",
            "hotkeys.restore_all": "<ctrl>+r",
            "hotkeys.toggle_thumbnails": "<ctrl>+t",
            "hotkeys.toggle_lock": "<ctrl>+l",
            "character_hotkeys": {},
            "hotkeys.cycle_next": "",  # Empty
            "hotkeys.cycle_prev": "",  # Empty
        }.get(key, default)

        window.hotkey_manager = MagicMock()

        MainWindowV21._register_hotkeys(window)

        # Should still complete without error
        window.logger.info.assert_called()


class TestRegisterCyclingHotkeys:
    """Tests for _register_cycling_hotkeys method"""

    def test_register_cycling_hotkeys_unregisters_old(self):
        """Test that old cycling hotkeys are unregistered before re-registering"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock()
        window.settings_manager = MagicMock()
        window.settings_manager.get.side_effect = lambda key, default=None: {
            "hotkeys.cycle_next": "<ctrl>+n",
            "hotkeys.cycle_prev": "<ctrl>+p",
        }.get(key, default)
        window.hotkey_manager = MagicMock()
        window.logger = MagicMock()

        MainWindowV21._register_cycling_hotkeys(window)

        window.hotkey_manager.unregister_hotkey.assert_any_call("cycle_next", restart=False)
        window.hotkey_manager.unregister_hotkey.assert_any_call("cycle_prev", restart=False)
        window.hotkey_manager.register_hotkey.assert_any_call(
            "cycle_next", "<ctrl>+n", window._cycle_next
        )
        window.hotkey_manager.register_hotkey.assert_any_call(
            "cycle_prev", "<ctrl>+p", window._cycle_prev
        )

    def test_register_cycling_hotkeys_skips_empty(self):
        """Test that empty combos are not registered"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock()
        window.settings_manager = MagicMock()
        window.settings_manager.get.side_effect = lambda key, default=None: {
            "hotkeys.cycle_next": "",
            "hotkeys.cycle_prev": "",
        }.get(key, default)
        window.hotkey_manager = MagicMock()
        window.logger = MagicMock()

        MainWindowV21._register_cycling_hotkeys(window)

        # Unregister should still be called
        assert window.hotkey_manager.unregister_hotkey.call_count == 2
        # But register should NOT be called (empty combos)
        window.hotkey_manager.register_hotkey.assert_not_called()


# Test _activate_window (platform abstraction layer)
class TestActivateWindowPlatform:
    """Tests for _activate_window method using capture_system abstraction"""

    def _make_window(self, *, auto_minimize=False, valid_id=True):
        """Helper to create a mock window with capture_system."""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = auto_minimize
        window.capture_system = MagicMock()
        window.capture_system._window_mgr = MagicMock()
        window.capture_system._window_mgr.is_valid_window_id.return_value = valid_id
        return window

    def test_activate_window_success(self):
        """Test activating window delegates to capture_system"""
        window = self._make_window()

        from argus_overview.ui.main_window_v21 import MainWindowV21

        MainWindowV21._activate_window(window, "0x12345")

        window.capture_system.activate_window.assert_called_once_with("0x12345")

    def test_activate_window_failure(self):
        """Test activate window handles failure"""
        window = self._make_window()
        window.capture_system.activate_window.side_effect = OSError("failed")

        from argus_overview.ui.main_window_v21 import MainWindowV21

        MainWindowV21._activate_window(window, "0x12345")

        window.logger.error.assert_called()

    def test_activate_window_with_auto_minimize(self):
        """Test activating window minimizes previous when auto-minimize enabled"""
        window = self._make_window(auto_minimize=True)
        window.settings_manager.get_last_activated_window.return_value = "0x99999"

        from argus_overview.ui.main_window_v21 import MainWindowV21

        MainWindowV21._activate_window(window, "0x12345")

        # Verify minimize was called on previous window
        window.capture_system.minimize_window.assert_called_once_with("0x99999")
        # Verify activate was called on new window
        window.capture_system.activate_window.assert_called_once_with("0x12345")

    def test_activate_window_invalid_id_none(self):
        """Test activate window rejects None window ID"""
        window = self._make_window(valid_id=False)

        from argus_overview.ui.main_window_v21 import MainWindowV21

        MainWindowV21._activate_window(window, None)

        window.logger.warning.assert_called()
        window.capture_system.activate_window.assert_not_called()

    def test_activate_window_invalid_id_format(self):
        """Test activate window rejects invalid window ID format"""
        window = self._make_window(valid_id=False)

        from argus_overview.ui.main_window_v21 import MainWindowV21

        # Test various invalid formats
        for invalid_id in ["12345", "abc", "0xGGGG", "", "window123"]:
            window.capture_system.activate_window.reset_mock()
            window.logger.reset_mock()

            MainWindowV21._activate_window(window, invalid_id)

            window.logger.warning.assert_called()
            window.capture_system.activate_window.assert_not_called()

    def test_activate_window_valid_id_formats(self):
        """Test activate window accepts valid window ID formats"""
        window = self._make_window()

        from argus_overview.ui.main_window_v21 import MainWindowV21

        # Test various valid formats
        for valid_id in ["0x12345", "0xABCDEF", "0x0", "0xFFFFFFFF"]:
            window.capture_system.activate_window.reset_mock()

            MainWindowV21._activate_window(window, valid_id)

            window.capture_system.activate_window.assert_called_once_with(valid_id)


# Test _create_menu_bar
class TestCreateMenuBar:
    """Tests for _create_menu_bar method"""

    @patch("argus_overview.ui.main_window_v21.MenuBuilder")
    @patch("argus_overview.ui.main_window_v21.ActionRegistry")
    def test_create_menu_bar(self, mock_registry_class, mock_builder_class):
        """Test creating menu bar"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.menuBar.return_value = MagicMock()

        mock_registry = MagicMock()
        mock_registry_class.get_instance.return_value = mock_registry

        mock_builder = MagicMock()
        mock_builder.build_help_menu.return_value = MagicMock()
        mock_builder_class.return_value = mock_builder

        MainWindowV21._create_menu_bar(window)

        # Should build help menu
        mock_builder.build_help_menu.assert_called_once()
        # Should add menu to menubar
        window.menuBar().addMenu.assert_called_once()


# Test cycling when character not found (covers 244-246, 266-268)
class TestCyclingCharNotFound:
    """Tests for cycling when character not found - covers recursive branches"""

    def test_cycle_next_char_not_found_logs_warning(self):
        """Test _cycle_next logs warning when character not found and no recursion"""
        window = create_mock_window()
        window.cycling_index = 0
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = {"Default": ["OnlyChar"]}
        window.current_cycling_group = "Default"

        # Set up main_tab with empty preview_frames so character not found
        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {}  # No windows

        # Mock _cycle_next to not recurse (avoid infinite loop in test)
        original_cycle_next = window._cycle_next
        call_count = [0]

        def cycle_next_once():
            call_count[0] += 1
            if call_count[0] == 1:
                original_cycle_next()

        window._cycle_next = cycle_next_once

        window._cycle_next()

        # Should log warning about character not found
        window.logger.warning.assert_called()

    def test_cycle_prev_char_not_found_logs_warning(self):
        """Test _cycle_prev logs warning when character not found and no recursion"""
        window = create_mock_window()
        window.cycling_index = 0
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = {"Default": ["OnlyChar"]}
        window.current_cycling_group = "Default"

        # Set up main_tab with empty preview_frames so character not found
        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {}  # No windows

        # Mock _cycle_prev to not recurse (avoid infinite loop in test)
        original_cycle_prev = window._cycle_prev
        call_count = [0]

        def cycle_prev_once():
            call_count[0] += 1
            if call_count[0] == 1:
                original_cycle_prev()

        window._cycle_prev = cycle_prev_once

        window._cycle_prev()

        # Should log warning about character not found
        window.logger.warning.assert_called()


# Test _create_system_tray
class TestCreateSystemTray:
    """Tests for _create_system_tray method"""

    @patch("argus_overview.ui.main_window_v21.SystemTray")
    def test_create_system_tray(self, mock_tray_class):
        """Test creating system tray"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.layout_manager = MagicMock()

        # Mock presets
        mock_preset1 = MagicMock()
        mock_preset1.name = "Profile1"
        mock_preset2 = MagicMock()
        mock_preset2.name = "Profile2"
        window.layout_manager.get_all_presets.return_value = [mock_preset1, mock_preset2]

        mock_tray = MagicMock()
        mock_tray_class.return_value = mock_tray

        MainWindowV21._create_system_tray(window)

        # Should create tray
        mock_tray_class.assert_called_once_with(window)

        # Should connect signals
        assert mock_tray.show_hide_requested.connect.called
        assert mock_tray.minimize_all_requested.connect.called

        # Should set profiles
        mock_tray.set_profiles.assert_called_once_with(["Profile1", "Profile2"])

        # Should show tray
        mock_tray.show.assert_called_once()


# Test _create_main_tab
class TestCreateMainTab:
    """Tests for _create_main_tab method"""

    @patch("argus_overview.ui.main_tab.MainTab")
    def test_create_main_tab(self, mock_tab_class):
        """Test creating main tab"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.capture_system = MagicMock()
        window.character_manager = MagicMock()
        window.settings_manager = MagicMock()
        window.tabs = MagicMock()

        mock_tab = MagicMock()
        mock_tab_class.return_value = mock_tab

        MainWindowV21._create_main_tab(window)

        # Should create tab with correct arguments
        mock_tab_class.assert_called_once()
        window.tabs.addTab.assert_called_once()

        # Should connect signals
        assert mock_tab.character_detected.connect.called
        assert mock_tab.layout_applied.connect.called


# Test _create_characters_tab
class TestCreateCharactersTab:
    """Tests for _create_characters_tab method"""

    @patch("argus_overview.ui.characters_teams_tab.CharactersTeamsTab")
    def test_create_characters_tab(self, mock_tab_class):
        """Test creating characters tab"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.character_manager = MagicMock()
        window.layout_manager = MagicMock()
        window.settings_sync = MagicMock()
        window.tabs = MagicMock()

        mock_tab = MagicMock()
        mock_tab_class.return_value = mock_tab

        MainWindowV21._create_characters_tab(window)

        # Should create tab
        mock_tab_class.assert_called_once()
        window.tabs.addTab.assert_called_once()

        # Should connect team_selected signal
        assert mock_tab.team_selected.connect.called


# Test _create_hotkeys_tab
class TestCreateHotkeysTab:
    """Tests for _create_hotkeys_tab method"""

    @patch("argus_overview.ui.hotkeys_tab.HotkeysTab")
    def test_create_hotkeys_tab(self, mock_tab_class):
        """Test creating hotkeys tab"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.character_manager = MagicMock()
        window.settings_manager = MagicMock()
        window.main_tab = MagicMock()
        window.tabs = MagicMock()
        window.hotkey_manager = MagicMock()

        mock_tab = MagicMock()
        mock_tab_class.return_value = mock_tab

        MainWindowV21._create_hotkeys_tab(window)

        # Should create tab
        mock_tab_class.assert_called_once()
        window.tabs.addTab.assert_called_once()

        # Should connect group_changed signal
        assert mock_tab.group_changed.connect.called

        # Should connect recording signals for pause/resume
        assert mock_tab.cycle_forward_edit.recordingStarted.connect.called
        assert mock_tab.cycle_backward_edit.recordingStopped.connect.called


# Test _create_settings_sync_tab
class TestCreateSettingsSyncTab:
    """Tests for _create_settings_sync_tab method"""

    @patch("argus_overview.ui.settings_sync_tab.SettingsSyncTab")
    def test_create_settings_sync_tab(self, mock_tab_class):
        """Test creating settings sync tab"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.settings_sync = MagicMock()
        window.character_manager = MagicMock()
        window.tabs = MagicMock()

        mock_tab = MagicMock()
        mock_tab_class.return_value = mock_tab

        MainWindowV21._create_settings_sync_tab(window)

        # Should create tab
        mock_tab_class.assert_called_once()
        window.tabs.addTab.assert_called_once()


# Test _create_settings_tab
class TestCreateSettingsTab:
    """Tests for _create_settings_tab method"""

    @patch("argus_overview.ui.settings_tab.SettingsTab")
    def test_create_settings_tab(self, mock_tab_class):
        """Test creating settings tab"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.settings_manager = MagicMock()
        window.hotkey_manager = MagicMock()
        window.tabs = MagicMock()

        mock_tab = MagicMock()
        mock_tab_class.return_value = mock_tab

        MainWindowV21._create_settings_tab(window)

        # Should create tab
        mock_tab_class.assert_called_once()
        window.tabs.addTab.assert_called_once()

        # Should connect settings_changed signal
        assert mock_tab.settings_changed.connect.called


# Test _apply_low_power_mode
class TestApplyLowPowerMode:
    """Tests for _apply_low_power_mode method"""

    def test_enable_low_power_mode(self):
        """Test enabling low power mode"""
        window = create_mock_window()
        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.refresh_rate_spin = MagicMock()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = 30  # previous FPS
        window.statusBar = MagicMock(return_value=MagicMock())

        window._apply_low_power_mode(True)

        # Should set FPS to 5
        window.main_tab.window_manager.set_refresh_rate.assert_called_with(5)
        # Should update spinner
        window.main_tab.refresh_rate_spin.setValue.assert_called_with(5)
        # Should show status message
        window.statusBar().showMessage.assert_called()

    def test_disable_low_power_mode(self):
        """Test disabling low power mode restores previous settings"""
        window = create_mock_window()
        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.refresh_rate_spin = MagicMock()
        window.settings_manager = MagicMock()
        window.statusBar = MagicMock(return_value=MagicMock())

        # Simulate that low power mode was previously enabled
        window._low_power_previous = {"fps": 30}

        window._apply_low_power_mode(False)

        # Should restore FPS to 30
        window.main_tab.window_manager.set_refresh_rate.assert_called_with(30)
        # Should update spinner
        window.main_tab.refresh_rate_spin.setValue.assert_called_with(30)
        # Should show status message
        window.statusBar().showMessage.assert_called()

    def test_enable_low_power_mode_stores_previous(self):
        """Test that enabling stores previous settings for restoration"""
        window = create_mock_window()
        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.refresh_rate_spin = MagicMock()
        window.settings_manager = MagicMock()
        window.settings_manager.get.side_effect = lambda k, d=None: {
            "performance.default_refresh_rate": 45,
        }.get(k, d)
        window.statusBar = MagicMock(return_value=MagicMock())

        window._apply_low_power_mode(True)

        # Should have stored previous values
        assert hasattr(window, "_low_power_previous")
        assert window._low_power_previous["fps"] == 45

    def test_disable_low_power_mode_without_previous(self):
        """Test disabling when no previous settings stored"""
        window = create_mock_window()
        window.main_tab = MagicMock()
        window.settings_manager = MagicMock()
        window.statusBar = MagicMock(return_value=MagicMock())

        # No _low_power_previous attribute
        window._apply_low_power_mode(False)

        # Should still show status message
        window.statusBar().showMessage.assert_called()

    def test_enable_low_power_no_main_tab(self):
        """Test enabling low power mode when main_tab doesn't exist"""
        window = create_mock_window()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = 30
        window.statusBar = MagicMock(return_value=MagicMock())

        # Remove main_tab attribute
        del window.main_tab

        # Should not raise
        window._apply_low_power_mode(True)

        # Should still show status
        window.statusBar().showMessage.assert_called()


# Test _apply_setting for low_power_mode
class TestApplySettingLowPowerMode:
    """Tests for _apply_setting with low_power_mode"""

    def test_apply_setting_low_power_mode_enabled(self):
        """Test applying low_power_mode setting (enabled)"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window._apply_low_power_mode = MagicMock()

        MainWindowV21._apply_setting(window, "performance.low_power_mode", True)

        window._apply_low_power_mode.assert_called_once_with(True)

    def test_apply_setting_low_power_mode_disabled(self):
        """Test applying low_power_mode setting (disabled)"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window._apply_low_power_mode = MagicMock()

        MainWindowV21._apply_setting(window, "performance.low_power_mode", False)

        window._apply_low_power_mode.assert_called_once_with(False)


# Test _apply_to_all_windows with invalid action
class TestApplyToAllWindowsInvalidAction:
    """Tests for _apply_to_all_windows with invalid action"""

    def test_apply_to_all_windows_invalid_action(self):
        """Test that invalid action name returns early without error"""
        window = create_mock_window()

        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {"0x111": MagicMock()}

        # Mock capture_system without the invalid method
        window.capture_system = MagicMock(spec=["minimize_window", "restore_window"])
        # getattr(capture_system, "invalid_window", None) will return None

        # Should not raise - just returns early
        window._apply_to_all_windows("invalid")

        # Should not try to call any methods since action was invalid
        window.capture_system.minimize_window.assert_not_called()
        window.capture_system.restore_window.assert_not_called()


# Test _create_intel_tab
class TestCreateIntelTab:
    """Tests for _create_intel_tab method"""

    @patch("argus_overview.ui.intel_tab.IntelTab")
    def test_create_intel_tab(self, mock_tab_class):
        """Test creating intel tab"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.settings_manager = MagicMock()
        window.tabs = MagicMock()

        mock_tab = MagicMock()
        mock_dispatcher = MagicMock()
        mock_tab.get_alert_dispatcher.return_value = mock_dispatcher
        mock_tab_class.return_value = mock_tab

        MainWindowV21._create_intel_tab(window)

        # Should create tab
        mock_tab_class.assert_called_once_with(window.settings_manager)
        window.tabs.addTab.assert_called_once()

        # Should connect signals
        assert mock_tab.alert_triggered.connect.called
        assert mock_tab.intel_received.connect.called
        assert mock_dispatcher.border_flash_requested.connect.called


# Test _flash_preview_borders
class TestFlashPreviewBorders:
    """Tests for _flash_preview_borders method"""

    def test_flash_preview_borders(self):
        """Test flashing preview borders"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)

        mock_frame1 = MagicMock()
        mock_frame2 = MagicMock()

        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {
            "0x111": mock_frame1,
            "0x222": mock_frame2,
        }

        MainWindowV21._flash_preview_borders(window, "#FF0000", 2000)

        mock_frame1.flash_border.assert_called_once_with("#FF0000", 2000)
        mock_frame2.flash_border.assert_called_once_with("#FF0000", 2000)

    def test_flash_preview_borders_no_main_tab(self):
        """Test flash_preview_borders handles missing main_tab"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        del window.main_tab

        # Should not raise
        MainWindowV21._flash_preview_borders(window, "#FF0000", 2000)

    def test_flash_preview_borders_no_window_manager(self):
        """Test flash_preview_borders handles missing window_manager"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.main_tab = MagicMock(spec=[])  # No window_manager attr

        # Should not raise
        MainWindowV21._flash_preview_borders(window, "#FF0000", 2000)

    def test_flash_preview_borders_frame_without_flash_border(self):
        """Test flash_preview_borders handles frames without flash_border method"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)

        mock_frame = MagicMock(spec=[])  # No flash_border method

        window.main_tab = MagicMock()
        window.main_tab.window_manager = MagicMock()
        window.main_tab.window_manager.preview_frames = {"0x111": mock_frame}

        # Should not raise
        MainWindowV21._flash_preview_borders(window, "#FF0000", 2000)


# Test _on_intel_alert
class TestOnIntelAlert:
    """Tests for _on_intel_alert method"""

    def test_on_intel_alert_critical_shows_notification(self):
        """Test critical intel alert shows tray notification"""
        from argus_overview.intel.parser import IntelReport, ThreatLevel
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.system_tray = MagicMock()

        report = IntelReport(
            system="HED-GP",
            threat_level=ThreatLevel.CRITICAL,
            hostile_count=5,
            ship_types=["Sabre", "Loki"],
            player_names=[],
            raw_message="HED-GP 5 hostiles",
        )

        MainWindowV21._on_intel_alert(window, report, MagicMock())

        window.system_tray.show_notification.assert_called_once()
        call_args = window.system_tray.show_notification.call_args[0]
        assert "CRITICAL" in call_args[0]
        assert "HED-GP" in call_args[0]

    def test_on_intel_alert_warning_no_notification(self):
        """Test warning-level alert doesn't show notification"""
        from argus_overview.intel.parser import IntelReport, ThreatLevel
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.system_tray = MagicMock()

        report = IntelReport(
            system="HED-GP",
            threat_level=ThreatLevel.WARNING,
            hostile_count=1,
            ship_types=[],
            player_names=[],
            raw_message="HED-GP 1 neutral",
        )

        MainWindowV21._on_intel_alert(window, report, MagicMock())

        window.system_tray.show_notification.assert_not_called()

    def test_on_intel_alert_invalid_report_type(self):
        """Test intel alert with non-IntelReport type returns early"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.system_tray = MagicMock()

        # Pass invalid type
        MainWindowV21._on_intel_alert(window, "not a report", MagicMock())

        window.system_tray.show_notification.assert_not_called()

    def test_on_intel_alert_no_system_tray(self):
        """Test intel alert handles missing system_tray"""
        from argus_overview.intel.parser import IntelReport, ThreatLevel
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        del window.system_tray  # No system tray

        report = IntelReport(
            system="HED-GP",
            threat_level=ThreatLevel.CRITICAL,
            hostile_count=5,
            ship_types=[],
            player_names=[],
            raw_message="test",
        )

        # Should not raise
        MainWindowV21._on_intel_alert(window, report, MagicMock())


# Test _on_intel_received
class TestOnIntelReceived:
    """Tests for _on_intel_received method"""

    def test_on_intel_received(self):
        """Test intel received handler (currently a pass)"""
        from argus_overview.intel.parser import IntelReport, ThreatLevel
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)

        report = IntelReport(
            system="TEST",
            threat_level=ThreatLevel.INFO,
            hostile_count=0,
            ship_types=[],
            player_names=[],
            raw_message="test",
        )

        # Should not raise (just a pass)
        MainWindowV21._on_intel_received(window, report)


# Test _disconnect_signals
class TestDisconnectSignals:
    """Tests for _disconnect_signals method"""

    def test_disconnect_signals_all_present(self):
        """Test disconnecting all signals when all tabs present"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()

        # Mock all tabs and their signals
        window.main_tab = MagicMock()
        window.characters_tab = MagicMock()
        window.settings_tab = MagicMock()
        window.hotkeys_tab = MagicMock()
        window.intel_tab = MagicMock()
        window.system_tray = MagicMock()
        window.auto_discovery = MagicMock()
        window.hotkey_manager = MagicMock()

        # Bind the required methods
        window._on_character_detected = MagicMock()
        window._on_layout_applied = MagicMock()
        window._on_team_selected = MagicMock()
        window._apply_setting = MagicMock()
        window._toggle_visibility = MagicMock()
        window._toggle_thumbnails = MagicMock()
        window._minimize_all_windows = MagicMock()
        window._restore_all_windows = MagicMock()
        window._on_profile_selected = MagicMock()
        window._show_settings = MagicMock()
        window._reload_config = MagicMock()
        window._quit_application = MagicMock()
        window._on_new_character_discovered = MagicMock()
        window._on_character_gone = MagicMock()
        window._on_intel_alert = MagicMock()
        window._on_intel_received = MagicMock()

        MainWindowV21._disconnect_signals(window)

        window.logger.debug.assert_called_with("Signals disconnected")

    def test_disconnect_signals_missing_tabs(self):
        """Test disconnecting signals when some tabs missing"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()

        # Only have main_tab and hotkey_manager
        window.main_tab = MagicMock()
        window.hotkey_manager = MagicMock()
        window._on_character_detected = MagicMock()
        window._on_layout_applied = MagicMock()

        # Remove other tabs
        del window.characters_tab
        del window.settings_tab
        del window.hotkeys_tab
        del window.intel_tab
        del window.system_tray
        del window.auto_discovery

        # Should not raise
        MainWindowV21._disconnect_signals(window)

        window.logger.debug.assert_called_with("Signals disconnected")

    def test_disconnect_signals_handles_runtime_error(self):
        """Test disconnecting handles RuntimeError (already disconnected)"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.hotkey_manager = MagicMock()

        # main_tab with signal that raises RuntimeError on disconnect
        window.main_tab = MagicMock()
        window.main_tab.character_detected = MagicMock()
        window.main_tab.character_detected.disconnect.side_effect = RuntimeError(
            "Already disconnected"
        )
        window.main_tab.layout_applied = MagicMock()

        window._on_character_detected = MagicMock()
        window._on_layout_applied = MagicMock()

        # Should not raise
        MainWindowV21._disconnect_signals(window)

        window.logger.debug.assert_called_with("Signals disconnected")

    def test_disconnect_signals_with_hotkeys_tab_edits(self):
        """Test disconnecting hotkey recording signals"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.hotkey_manager = MagicMock()

        # hotkeys_tab with recording edits
        window.hotkeys_tab = MagicMock()
        window.hotkeys_tab.cycle_forward_edit = MagicMock()
        window.hotkeys_tab.cycle_backward_edit = MagicMock()
        window.hotkeys_tab.group_changed = MagicMock()
        window.main_tab = MagicMock()

        window._on_character_detected = MagicMock()
        window._on_layout_applied = MagicMock()

        MainWindowV21._disconnect_signals(window)

        # Should have tried to disconnect recording signals
        window.hotkeys_tab.cycle_forward_edit.recordingStarted.disconnect.assert_called()
        window.hotkeys_tab.cycle_backward_edit.recordingStopped.disconnect.assert_called()

    def test_disconnect_signals_no_cycle_edits(self):
        """Test disconnecting when cycle edits don't exist"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.hotkey_manager = MagicMock()

        # hotkeys_tab without cycle edits
        window.hotkeys_tab = MagicMock()
        window.hotkeys_tab.cycle_forward_edit = None
        window.hotkeys_tab.cycle_backward_edit = None
        window.hotkeys_tab.group_changed = MagicMock()
        window.main_tab = MagicMock()

        window._on_character_detected = MagicMock()
        window._on_layout_applied = MagicMock()

        # Should not raise
        MainWindowV21._disconnect_signals(window)


# Test _on_character_gone
class TestOnCharacterGone:
    """Tests for _on_character_gone method"""

    def test_on_character_gone_unassigns_window(self):
        """Test character gone unassigns window from character manager"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.character_manager = MagicMock()

        MainWindowV21._on_character_gone(window, "TestPilot", "0x12345")

        window.character_manager.unassign_window.assert_called_with("TestPilot")
        window.logger.info.assert_called()

    def test_on_character_gone_updates_characters_tab(self):
        """Test character gone updates characters tab status"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.character_manager = MagicMock()
        window.characters_tab = MagicMock()

        MainWindowV21._on_character_gone(window, "TestPilot", "0x12345")

        window.characters_tab.update_character_status.assert_called_with("TestPilot", None)

    def test_on_character_gone_no_characters_tab(self):
        """Test character gone when no characters_tab"""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.character_manager = MagicMock()
        del window.characters_tab

        # Should not raise
        MainWindowV21._on_character_gone(window, "Pilot", "0x12345")

        window.character_manager.unassign_window.assert_called_with("Pilot")


class TestDisconnectSignalsRecordingException:
    """Tests for exception handling in recording signal disconnect (lines 752-753)."""

    def test_disconnect_signals_recording_type_error(self):
        """Test disconnecting handles TypeError on recording signal disconnect."""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.hotkey_manager = MagicMock()
        window.main_tab = MagicMock()
        window._on_character_detected = MagicMock()
        window._on_layout_applied = MagicMock()

        # hotkeys_tab with recording edits that raise TypeError
        window.hotkeys_tab = MagicMock()
        mock_edit = MagicMock()
        mock_edit.recordingStarted = MagicMock()
        mock_edit.recordingStarted.disconnect.side_effect = TypeError("Not connected")
        mock_edit.recordingStopped = MagicMock()
        mock_edit.recordingStopped.disconnect.side_effect = RuntimeError("Already disconnected")
        window.hotkeys_tab.cycle_forward_edit = mock_edit
        window.hotkeys_tab.cycle_backward_edit = mock_edit
        window.hotkeys_tab.group_changed = MagicMock()

        # Should not raise - exceptions are caught
        MainWindowV21._disconnect_signals(window)

        window.logger.debug.assert_called_with("Signals disconnected")


class TestCloseEventMainTab:
    """Tests for closeEvent main_tab handling (line 933)."""

    def test_close_event_stops_main_tab_capture_loop(self, qapp):
        """Test closeEvent calls main_tab.stop_capture_loop."""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = False  # Don't minimize to tray
        window.main_tab = MagicMock()
        window.intel_tab = MagicMock()
        window.auto_discovery = MagicMock()
        window.capture_system = MagicMock()
        window.hotkey_manager = MagicMock()
        window.system_tray = MagicMock()
        window._disconnect_signals = MagicMock()
        window._already_closing = False

        # Call closeEvent
        from PySide6.QtGui import QCloseEvent

        event = MagicMock(spec=QCloseEvent)
        MainWindowV21.closeEvent(window, event)

        window.main_tab.stop_capture_loop.assert_called_once()


class TestCloseEventIntelTab:
    """Tests for closeEvent intel_tab handling (line 937)."""

    def test_close_event_stops_intel_tab(self, qapp):
        """Test closeEvent calls intel_tab.stop."""
        from argus_overview.ui.main_window_v21 import MainWindowV21

        window = MagicMock(spec=MainWindowV21)
        window.logger = MagicMock()
        window.settings_manager = MagicMock()
        window.settings_manager.get.return_value = False  # Don't minimize to tray
        window.main_tab = MagicMock()
        window.intel_tab = MagicMock()
        window.auto_discovery = MagicMock()
        window.capture_system = MagicMock()
        window.hotkey_manager = MagicMock()
        window.system_tray = MagicMock()
        window._disconnect_signals = MagicMock()
        window._already_closing = False

        # Call closeEvent
        from PySide6.QtGui import QCloseEvent

        event = MagicMock(spec=QCloseEvent)
        MainWindowV21.closeEvent(window, event)

        window.intel_tab.stop.assert_called_once()


# =============================================================================
# Coverage Push: __init__ partial (lines 72-154, ~45 lines)
# =============================================================================


class TestMainWindowV21InitPartial:
    """Tests for MainWindowV21.__init__ initialization sequence.

    Patches all Qt and core dependencies to test the init sequence
    without requiring a real display or full Qt application.
    """

    def test_init_creates_core_modules(self):
        """Test __init__ creates all core subsystems"""
        from contextlib import ExitStack

        from argus_overview.ui.main_window_v21 import MainWindowV21

        mock_char_mgr = MagicMock()
        mock_layout_mgr = MagicMock()
        mock_hotkey_mgr = MagicMock()
        mock_settings_sync = MagicMock()
        mock_settings_mgr = MagicMock()
        mock_settings_mgr.get.return_value = 1
        mock_settings_mgr.validate = MagicMock()
        mock_capture = MagicMock()
        mock_discovery = MagicMock()
        mock_theme = MagicMock()

        mod = "argus_overview.ui.main_window_v21"
        with ExitStack() as stack:
            stack.enter_context(patch(f"{mod}.CharacterManager", return_value=mock_char_mgr))
            stack.enter_context(patch(f"{mod}.LayoutManager", return_value=mock_layout_mgr))
            stack.enter_context(patch(f"{mod}.HotkeyManager", return_value=mock_hotkey_mgr))
            stack.enter_context(patch(f"{mod}.EVESettingsSync", return_value=mock_settings_sync))
            stack.enter_context(patch(f"{mod}.SettingsManager", return_value=mock_settings_mgr))
            stack.enter_context(patch(f"{mod}.WindowCaptureThreaded", return_value=mock_capture))
            stack.enter_context(patch(f"{mod}.AutoDiscovery", return_value=mock_discovery))
            stack.enter_context(patch(f"{mod}.get_theme_manager", return_value=mock_theme))
            # Patch QMainWindow.__init__ to skip real Qt initialization
            stack.enter_context(patch("PySide6.QtWidgets.QMainWindow.__init__", return_value=None))
            stack.enter_context(patch.object(MainWindowV21, "setWindowTitle"))
            stack.enter_context(patch.object(MainWindowV21, "setMinimumSize"))
            stack.enter_context(patch.object(MainWindowV21, "setCentralWidget"))
            stack.enter_context(patch.object(MainWindowV21, "_set_window_icon"))
            stack.enter_context(patch.object(MainWindowV21, "_apply_initial_settings"))
            stack.enter_context(patch.object(MainWindowV21, "_create_menu_bar"))
            stack.enter_context(patch.object(MainWindowV21, "_create_main_tab"))
            stack.enter_context(patch.object(MainWindowV21, "_create_hotkeys_tab"))
            stack.enter_context(patch.object(MainWindowV21, "_create_characters_tab"))
            stack.enter_context(patch.object(MainWindowV21, "_create_intel_tab"))
            stack.enter_context(patch.object(MainWindowV21, "_create_settings_sync_tab"))
            stack.enter_context(patch.object(MainWindowV21, "_create_settings_tab"))
            stack.enter_context(patch.object(MainWindowV21, "_connect_signals"))
            stack.enter_context(patch.object(MainWindowV21, "_create_system_tray"))
            stack.enter_context(patch.object(MainWindowV21, "_register_hotkeys"))
            stack.enter_context(patch(f"{mod}.QTabWidget"))
            stack.enter_context(patch(f"{mod}.QVBoxLayout"))
            stack.enter_context(patch(f"{mod}.QWidget"))

            window = MainWindowV21()

            # Core modules created
            assert window.character_manager is mock_char_mgr
            assert window.layout_manager is mock_layout_mgr
            assert window.hotkey_manager is mock_hotkey_mgr
            assert window.settings_sync is mock_settings_sync
            assert window.settings_manager is mock_settings_mgr
            assert window.capture_system is mock_capture
            assert window.auto_discovery is mock_discovery
            assert window.theme_manager is mock_theme

            # Cycling state initialized
            assert window.cycling_index == 0
            assert window.current_cycling_group == "Default"

            # Systems started
            mock_capture.start.assert_called_once()
            mock_hotkey_mgr.start.assert_called_once()

            # Settings validated
            mock_settings_mgr.validate.assert_called_once()

            # Theme applied
            mock_theme.apply_theme.assert_called()



"""
Settings Tab - Application settings configuration
Configure all application settings with category-based panels
"""

import logging
from typing import Any

from PySide6.QtCore import Qt, Signal
from PySide6.QtGui import QColor, QFont
from PySide6.QtWidgets import (
    QCheckBox,
    QColorDialog,
    QComboBox,
    QDialog,
    QDialogButtonBox,
    QFormLayout,
    QGroupBox,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
    QSpinBox,
    QSplitter,
    QStackedWidget,
    QTableWidget,
    QTableWidgetItem,
    QTreeWidget,
    QTreeWidgetItem,
    QVBoxLayout,
    QWidget,
)


class HotkeyEditDialog(QDialog):
    """Dialog for editing hotkey bindings"""

    def __init__(self, action: str, current_combo: str, hotkey_manager, parent=None):
        super().__init__(parent)
        self.action = action
        self.current_combo = current_combo
        self.hotkey_manager = hotkey_manager
        self.logger = logging.getLogger(__name__)

        self.setWindowTitle(f"Edit Hotkey: {action}")
        self.setMinimumWidth(400)
        self._setup_ui()

    def _setup_ui(self):
        layout = QVBoxLayout()

        # Action label
        action_label = QLabel(f"<b>Action:</b> {self.action}")
        layout.addWidget(action_label)

        # Current binding
        current_label = QLabel(f"Current: {self.current_combo}")
        current_label.setStyleSheet("color: #888;")
        layout.addWidget(current_label)

        # Key input
        self.key_edit = QLineEdit()
        self.key_edit.setPlaceholderText("Enter hotkey (e.g., Ctrl+Alt+1)")
        self.key_edit.setText(self.current_combo)
        layout.addWidget(self.key_edit)

        # Format hint
        hint_label = QLabel(
            "Format: <ctrl>+<alt>+<key> or <shift>+<key>\nExamples: <ctrl>+<alt>+1, <shift>+f5"
        )
        hint_label.setStyleSheet("color: #666; font-size: 9pt;")
        hint_label.setWordWrap(True)
        layout.addWidget(hint_label)

        # Test button
        test_btn = QPushButton("Test Hotkey")
        test_btn.clicked.connect(self._test_hotkey)
        layout.addWidget(test_btn)

        # Conflict check label
        self.conflict_label = QLabel("")
        self.conflict_label.setStyleSheet("color: #e74c3c;")
        layout.addWidget(self.conflict_label)

        # Buttons
        button_box = QDialogButtonBox(
            QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel
        )
        button_box.accepted.connect(self._save_hotkey)
        button_box.rejected.connect(self.reject)
        layout.addWidget(button_box)

        self.setLayout(layout)

    def _test_hotkey(self):
        """Test the hotkey binding"""
        combo = self.key_edit.text().strip()
        if not combo:
            QMessageBox.warning(self, "Invalid", "Please enter a hotkey.")
            return

        QMessageBox.information(
            self,
            "Test Hotkey",
            f"Hotkey '{combo}' would be registered for action '{self.action}'.\n\n"
            "Note: Actual testing requires hotkey manager integration.",
        )

    def _save_hotkey(self):
        """Validate and save hotkey"""
        combo = self.key_edit.text().strip()
        if not combo:
            QMessageBox.warning(self, "Invalid", "Please enter a hotkey.")
            return

        # Basic validation
        if not ("<" in combo and ">" in combo):
            QMessageBox.warning(
                self,
                "Invalid Format",
                "Hotkey must be in format: <modifier>+<key>\nExample: <ctrl>+<alt>+1",
            )
            return

        self.accept()

    def get_hotkey(self) -> str:
        """Get the configured hotkey"""
        return self.key_edit.text().strip()


# Settings Panels


class GeneralPanel(QWidget):
    """General application settings"""

    setting_changed = Signal(str, object)

    def __init__(self, settings_manager):
        super().__init__()
        self.settings_manager = settings_manager
        self._setup_ui()

    def _setup_ui(self):
        layout = QVBoxLayout()

        group = QGroupBox("General Settings")
        form = QFormLayout()

        # Start with system
        self.start_system_check = QCheckBox()
        self.start_system_check.setChecked(
            self.settings_manager.get("general.start_with_system", False)
        )
        self.start_system_check.stateChanged.connect(
            lambda: self.setting_changed.emit(
                "general.start_with_system", self.start_system_check.isChecked()
            )
        )
        form.addRow("Start with system:", self.start_system_check)

        # Minimize to tray
        self.minimize_tray_check = QCheckBox()
        self.minimize_tray_check.setChecked(
            self.settings_manager.get("general.minimize_to_tray", True)
        )
        self.minimize_tray_check.stateChanged.connect(
            lambda: self.setting_changed.emit(
                "general.minimize_to_tray", self.minimize_tray_check.isChecked()
            )
        )
        form.addRow("Minimize to tray:", self.minimize_tray_check)

        # Show notifications
        self.notifications_check = QCheckBox()
        self.notifications_check.setChecked(
            self.settings_manager.get("general.show_notifications", True)
        )
        self.notifications_check.stateChanged.connect(
            lambda: self.setting_changed.emit(
                "general.show_notifications", self.notifications_check.isChecked()
            )
        )
        form.addRow("Show notifications:", self.notifications_check)

        # Auto-save interval
        self.auto_save_spin = QSpinBox()
        self.auto_save_spin.setRange(1, 60)
        self.auto_save_spin.setValue(self.settings_manager.get("general.auto_save_interval", 5))
        self.auto_save_spin.setSuffix(" min")
        self.auto_save_spin.valueChanged.connect(
            lambda v: self.setting_changed.emit("general.auto_save_interval", v)
        )
        form.addRow("Auto-save interval:", self.auto_save_spin)

        group.setLayout(form)
        layout.addWidget(group)
        layout.addStretch()
        self.setLayout(layout)


class PerformancePanel(QWidget):
    """Performance settings"""

    setting_changed = Signal(str, object)

    def __init__(self, settings_manager):
        super().__init__()
        self.settings_manager = settings_manager
        self._setup_ui()

    def _setup_ui(self):
        layout = QVBoxLayout()

        group = QGroupBox("Performance Settings")
        form = QFormLayout()

        # Low Power Mode (master toggle)
        self.low_power_check = QCheckBox()
        self.low_power_check.setChecked(
            self.settings_manager.get("performance.low_power_mode", False)
        )
        self.low_power_check.stateChanged.connect(self._on_low_power_changed)
        self.low_power_check.setToolTip(
            "LOW POWER MODE\n"
            "• Sets FPS to 5\n"
            "• Reduces CPU/GPU load significantly\n\n"
            "Use when running multiple EVE clients."
        )
        self.low_power_check.setStyleSheet("QCheckBox { font-weight: bold; color: #e67e22; }")
        form.addRow("⚡ Low Power Mode:", self.low_power_check)

        # Disable previews (GPU/CPU saver)
        self.disable_previews_check = QCheckBox()
        self.disable_previews_check.setChecked(
            self.settings_manager.get("performance.disable_previews", False)
        )
        self.disable_previews_check.stateChanged.connect(
            lambda: self.setting_changed.emit(
                "performance.disable_previews", self.disable_previews_check.isChecked()
            )
        )
        self.disable_previews_check.setToolTip(
            "Disable all window preview captures.\n"
            "Significantly reduces GPU/CPU usage.\n"
            "Window cycling and hotkeys still work."
        )
        form.addRow("Disable previews:", self.disable_previews_check)

        # Refresh rate
        self.refresh_rate_spin = QSpinBox()
        self.refresh_rate_spin.setRange(1, 60)
        self.refresh_rate_spin.setValue(
            self.settings_manager.get("performance.default_refresh_rate", 30)
        )
        self.refresh_rate_spin.setSuffix(" FPS")
        self.refresh_rate_spin.valueChanged.connect(
            lambda v: self.setting_changed.emit("performance.default_refresh_rate", v)
        )
        form.addRow("Default refresh rate:", self.refresh_rate_spin)

        # Capture workers
        self.workers_spin = QSpinBox()
        self.workers_spin.setRange(1, 16)
        self.workers_spin.setValue(self.settings_manager.get("performance.capture_workers", 4))
        self.workers_spin.valueChanged.connect(
            lambda v: self.setting_changed.emit("performance.capture_workers", v)
        )
        form.addRow("Capture workers:", self.workers_spin)

        # Caching
        self.caching_check = QCheckBox()
        self.caching_check.setChecked(self.settings_manager.get("performance.enable_caching", True))
        self.caching_check.stateChanged.connect(
            lambda: self.setting_changed.emit(
                "performance.enable_caching", self.caching_check.isChecked()
            )
        )
        form.addRow("Enable caching:", self.caching_check)

        # Cache size
        self.cache_size_spin = QSpinBox()
        self.cache_size_spin.setRange(10, 1000)
        self.cache_size_spin.setValue(self.settings_manager.get("performance.cache_size_mb", 100))
        self.cache_size_spin.setSuffix(" MB")
        self.cache_size_spin.valueChanged.connect(
            lambda v: self.setting_changed.emit("performance.cache_size_mb", v)
        )
        form.addRow("Cache size:", self.cache_size_spin)

        # Capture quality
        self.quality_combo = QComboBox()
        self.quality_combo.addItems(["low", "medium", "high"])
        current_quality = self.settings_manager.get("performance.capture_quality", "medium")
        self.quality_combo.setCurrentText(current_quality)
        self.quality_combo.currentTextChanged.connect(
            lambda v: self.setting_changed.emit("performance.capture_quality", v)
        )
        form.addRow("Capture quality:", self.quality_combo)

        group.setLayout(form)
        layout.addWidget(group)
        layout.addStretch()
        self.setLayout(layout)

    def _on_low_power_changed(self):
        """Handle low power mode toggle - emits setting change"""
        enabled = self.low_power_check.isChecked()
        self.setting_changed.emit("performance.low_power_mode", enabled)


class HotkeysPanel(QWidget):
    """Hotkey settings"""

    setting_changed = Signal(str, object)

    def __init__(self, settings_manager, hotkey_manager):
        super().__init__()
        self.settings_manager = settings_manager
        self.hotkey_manager = hotkey_manager
        self.logger = logging.getLogger(__name__)
        self._setup_ui()
        self._load_hotkeys()

    def _setup_ui(self):
        layout = QVBoxLayout()

        # Format label
        format_label = QLabel("Format: <ctrl>+<alt>+<key> or <shift>+<key>")
        format_label.setStyleSheet("color: #666; font-style: italic;")
        layout.addWidget(format_label)

        # Hotkeys table
        self.hotkeys_table = QTableWidget()
        self.hotkeys_table.setColumnCount(3)
        self.hotkeys_table.setHorizontalHeaderLabels(["Action", "Hotkey", "Status"])
        self.hotkeys_table.horizontalHeader().setStretchLastSection(False)
        self.hotkeys_table.setColumnWidth(0, 200)
        self.hotkeys_table.setColumnWidth(1, 150)
        self.hotkeys_table.setColumnWidth(2, 80)
        self.hotkeys_table.setSelectionBehavior(QTableWidget.SelectionBehavior.SelectRows)
        layout.addWidget(self.hotkeys_table)

        # Buttons
        buttons_layout = QHBoxLayout()

        edit_btn = QPushButton("Edit")
        edit_btn.clicked.connect(self._edit_hotkey)
        buttons_layout.addWidget(edit_btn)

        reset_btn = QPushButton("Reset to Default")
        reset_btn.clicked.connect(self._reset_hotkey)
        buttons_layout.addWidget(reset_btn)

        buttons_layout.addStretch()
        layout.addLayout(buttons_layout)

        self.setLayout(layout)

    def _load_hotkeys(self):
        """Load hotkeys from settings"""
        hotkeys = self.settings_manager.get("hotkeys", {})

        self.hotkeys_table.setRowCount(len(hotkeys))
        for row, (action, combo) in enumerate(hotkeys.items()):
            # Action
            action_item = QTableWidgetItem(action.replace("_", " ").title())
            self.hotkeys_table.setItem(row, 0, action_item)

            # Hotkey
            hotkey_item = QTableWidgetItem(combo)
            self.hotkeys_table.setItem(row, 1, hotkey_item)

            # Status
            status_item = QTableWidgetItem("Active")
            status_item.setForeground(QColor(0, 200, 0))
            self.hotkeys_table.setItem(row, 2, status_item)

    def _edit_hotkey(self):
        """Edit selected hotkey"""
        selected = self.hotkeys_table.selectedItems()
        if not selected:
            QMessageBox.information(self, "No Selection", "Please select a hotkey to edit.")
            return

        row = self.hotkeys_table.currentRow()
        action = self.hotkeys_table.item(row, 0).text()
        current_combo = self.hotkeys_table.item(row, 1).text()

        dialog = HotkeyEditDialog(action, current_combo, self.hotkey_manager, self)
        if dialog.exec():
            new_combo = dialog.get_hotkey()
            self.hotkeys_table.item(row, 1).setText(new_combo)

            # Save to settings
            action_key = action.lower().replace(" ", "_")
            self.setting_changed.emit(f"hotkeys.{action_key}", new_combo)

    def _reset_hotkey(self):
        """Reset selected hotkey to default"""
        selected = self.hotkeys_table.selectedItems()
        if not selected:
            QMessageBox.information(self, "No Selection", "Please select a hotkey to reset.")
            return

        QMessageBox.information(
            self, "Feature", "Reset to default will be implemented in a future update."
        )


class AppearancePanel(QWidget):
    """Appearance settings"""

    setting_changed = Signal(str, object)

    def __init__(self, settings_manager):
        super().__init__()
        self.settings_manager = settings_manager
        self._setup_ui()

    def _setup_ui(self):
        layout = QVBoxLayout()

        group = QGroupBox("Appearance Settings")
        form = QFormLayout()

        # Theme
        self.theme_combo = QComboBox()
        self.theme_combo.addItems(["dark", "light", "system"])
        self.theme_combo.setCurrentText(self.settings_manager.get("appearance.theme", "dark"))
        self.theme_combo.currentTextChanged.connect(
            lambda v: self.setting_changed.emit("appearance.theme", v)
        )
        form.addRow("Theme:", self.theme_combo)

        # Font size
        self.font_size_spin = QSpinBox()
        self.font_size_spin.setRange(8, 16)
        self.font_size_spin.setValue(self.settings_manager.get("appearance.font_size", 10))
        self.font_size_spin.setSuffix(" pt")
        self.font_size_spin.valueChanged.connect(
            lambda v: self.setting_changed.emit("appearance.font_size", v)
        )
        form.addRow("Font size:", self.font_size_spin)

        # Compact mode
        self.compact_check = QCheckBox()
        self.compact_check.setChecked(self.settings_manager.get("appearance.compact_mode", False))
        self.compact_check.stateChanged.connect(
            lambda: self.setting_changed.emit(
                "appearance.compact_mode", self.compact_check.isChecked()
            )
        )
        form.addRow("Compact mode:", self.compact_check)

        # Accent color
        accent_layout = QHBoxLayout()
        self.accent_color = self.settings_manager.get("appearance.accent_color", "#4287f5")
        self.accent_btn = QPushButton()
        self.accent_btn.setStyleSheet(f"background-color: {self.accent_color}; min-height: 30px;")
        self.accent_btn.clicked.connect(self._pick_accent_color)
        accent_layout.addWidget(self.accent_btn)
        accent_layout.addStretch()

        form.addRow("Accent color:", accent_layout)

        group.setLayout(form)
        layout.addWidget(group)
        layout.addStretch()
        self.setLayout(layout)

    def _pick_accent_color(self):
        """Pick accent color"""
        color = QColorDialog.getColor(QColor(self.accent_color), self, "Select Accent Color")
        if color.isValid():
            self.accent_color = color.name()
            self.accent_btn.setStyleSheet(
                f"background-color: {self.accent_color}; min-height: 30px;"
            )
            self.setting_changed.emit("appearance.accent_color", self.accent_color)


class AdvancedPanel(QWidget):
    """Advanced settings"""

    setting_changed = Signal(str, object)

    def __init__(self, settings_manager):
        super().__init__()
        self.settings_manager = settings_manager
        self._setup_ui()

    def _setup_ui(self):
        layout = QVBoxLayout()

        group = QGroupBox("Advanced Settings")
        form = QFormLayout()

        # Log level
        self.log_level_combo = QComboBox()
        self.log_level_combo.addItems(["DEBUG", "INFO", "WARNING", "ERROR"])
        self.log_level_combo.setCurrentText(self.settings_manager.get("advanced.log_level", "INFO"))
        self.log_level_combo.currentTextChanged.connect(
            lambda v: self.setting_changed.emit("advanced.log_level", v)
        )
        form.addRow("Log level:", self.log_level_combo)

        # Config directory
        self.config_dir_label = QLabel(
            self.settings_manager.get("advanced.config_directory", "~/.config/argus-overview")
        )
        self.config_dir_label.setStyleSheet("color: #888;")
        form.addRow("Config directory:", self.config_dir_label)

        # Enable debug
        self.debug_check = QCheckBox()
        self.debug_check.setChecked(self.settings_manager.get("advanced.enable_debug", False))
        self.debug_check.stateChanged.connect(
            lambda: self.setting_changed.emit("advanced.enable_debug", self.debug_check.isChecked())
        )
        form.addRow("Enable debug:", self.debug_check)

        group.setLayout(form)
        layout.addWidget(group)

        # Action buttons
        actions_group = QGroupBox("Actions")
        actions_layout = QVBoxLayout()

        clear_cache_btn = QPushButton("Clear Cache")
        clear_cache_btn.clicked.connect(self._clear_cache)
        actions_layout.addWidget(clear_cache_btn)

        export_btn = QPushButton("Export Settings")
        export_btn.clicked.connect(self._export_settings)
        actions_layout.addWidget(export_btn)

        import_btn = QPushButton("Import Settings")
        import_btn.clicked.connect(self._import_settings)
        actions_layout.addWidget(import_btn)

        actions_group.setLayout(actions_layout)
        layout.addWidget(actions_group)

        layout.addStretch()
        self.setLayout(layout)

    def _clear_cache(self):
        """Clear application cache"""
        QMessageBox.information(
            self, "Clear Cache", "Cache clearing will be implemented in a future update."
        )

    def _export_settings(self):
        """Export settings to file"""
        from PySide6.QtWidgets import QFileDialog

        filename, _ = QFileDialog.getSaveFileName(
            self, "Export Settings", "", "JSON Files (*.json)"
        )
        if filename:
            try:
                from pathlib import Path

                self.settings_manager.export_config(Path(filename))
                QMessageBox.information(self, "Success", f"Settings exported to {filename}")
            except (OSError, TypeError, ValueError) as e:
                QMessageBox.critical(self, "Error", f"Failed to export settings: {e}")

    def _import_settings(self):
        """Import settings from file"""
        from PySide6.QtWidgets import QFileDialog

        filename, _ = QFileDialog.getOpenFileName(
            self, "Import Settings", "", "JSON Files (*.json)"
        )
        if filename:
            try:
                from pathlib import Path

                self.settings_manager.import_config(Path(filename))
                QMessageBox.information(
                    self,
                    "Success",
                    f"Settings imported from {filename}\nRestart required to apply all changes.",
                )
            except (OSError, ValueError) as e:
                QMessageBox.critical(self, "Error", f"Failed to import settings: {e}")


class SettingsTab(QWidget):
    """Main Settings Tab widget"""

    settings_changed = Signal(str, object)  # key, value

    def __init__(self, settings_manager, hotkey_manager):
        super().__init__()
        self.settings_manager = settings_manager
        self.hotkey_manager = hotkey_manager
        self.logger = logging.getLogger(__name__)

        self._setup_ui()
        self._load_settings()

    def _setup_ui(self):
        layout = QHBoxLayout()

        # Splitter
        splitter = QSplitter(Qt.Orientation.Horizontal)

        # Left: Category tree
        left_panel = self._create_category_tree()
        splitter.addWidget(left_panel)

        # Right: Stacked panels
        self.panel_stack = QStackedWidget()

        # Create panels
        self.general_panel = GeneralPanel(self.settings_manager)
        self.general_panel.setting_changed.connect(self._on_setting_changed)
        self.panel_stack.addWidget(self.general_panel)

        self.performance_panel = PerformancePanel(self.settings_manager)
        self.performance_panel.setting_changed.connect(self._on_setting_changed)
        self.panel_stack.addWidget(self.performance_panel)

        self.hotkeys_panel = HotkeysPanel(self.settings_manager, self.hotkey_manager)
        self.hotkeys_panel.setting_changed.connect(self._on_setting_changed)
        self.panel_stack.addWidget(self.hotkeys_panel)

        self.appearance_panel = AppearancePanel(self.settings_manager)
        self.appearance_panel.setting_changed.connect(self._on_setting_changed)
        self.panel_stack.addWidget(self.appearance_panel)

        self.advanced_panel = AdvancedPanel(self.settings_manager)
        self.advanced_panel.setting_changed.connect(self._on_setting_changed)
        self.panel_stack.addWidget(self.advanced_panel)

        splitter.addWidget(self.panel_stack)

        splitter.setSizes([200, 600])
        layout.addWidget(splitter)

        self.setLayout(layout)

    def _create_category_tree(self) -> QWidget:
        """Create category tree widget"""
        widget = QWidget()
        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)

        title = QLabel("Settings")
        font = QFont()
        font.setPointSize(12)
        font.setBold(True)
        title.setFont(font)
        layout.addWidget(title)

        self.category_tree = QTreeWidget()
        self.category_tree.setHeaderHidden(True)

        categories = ["General", "Performance", "Hotkeys", "Appearance", "Advanced"]

        for category in categories:
            item = QTreeWidgetItem([category])
            self.category_tree.addTopLevelItem(item)

        self.category_tree.currentItemChanged.connect(self._on_category_changed)
        layout.addWidget(self.category_tree)

        # Reset button
        reset_btn = QPushButton("Reset All to Defaults")
        reset_btn.clicked.connect(self._reset_all)
        layout.addWidget(reset_btn)

        widget.setLayout(layout)
        return widget

    def _on_category_changed(self, current, previous):
        """Handle category change"""
        if current:
            categories = {
                "General": 0,
                "Performance": 1,
                "Hotkeys": 2,
                "Appearance": 3,
                "Advanced": 4,
            }
            category = current.text(0)
            if category in categories:
                self.panel_stack.setCurrentIndex(categories[category])

    def _load_settings(self):
        """Load settings from SettingsManager"""
        # Settings are loaded by individual panels
        pass

    def _on_setting_changed(self, key: str, value: Any):
        """Handle setting change"""
        self.logger.info(f"Setting changed: {key} = {value}")

        # Save to SettingsManager
        self.settings_manager.set(key, value)

        # Emit signal for main window to apply
        self.settings_changed.emit(key, value)

    def _reset_all(self):
        """Reset all settings to defaults"""
        reply = QMessageBox.question(
            self,
            "Confirm Reset",
            "Reset all settings to defaults?\n\nThis action cannot be undone.",
            QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
        )

        if reply == QMessageBox.StandardButton.Yes:
            self.settings_manager.reset_to_defaults()
            QMessageBox.information(
                self,
                "Reset Complete",
                "All settings have been reset to defaults.\nRestart required to apply changes.",
            )



"""
Intel Parser - Analyzes EVE chat messages for intel content.

Parses messages for:
- System names (null-sec format like HED-GP, and known systems)
- Hostile indicators (hostile, neut, red, spike, etc.)
- Ship types
- Player counts (+5, x3, gang of 10, etc.)
- Clear reports (clr, clear, nv, etc.)
"""

import logging
import re
from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .jumps import JumpCalculator


class ThreatLevel(Enum):
    """Threat level classification for intel reports."""

    CLEAR = "clear"  # System reported clear
    INFO = "info"  # General intel, not immediate threat
    WARNING = "warning"  # Hostiles nearby (2+ jumps)
    DANGER = "danger"  # Hostiles close (1 jump) or small gang
    CRITICAL = "critical"  # Hostiles in system or capital ships


@dataclass
class IntelReport:
    """Represents parsed intel from a chat message."""

    system: str | None
    threat_level: ThreatLevel
    hostile_count: int | None
    ship_types: list[str]
    player_names: list[str]
    raw_message: str
    timestamp: datetime = field(default_factory=datetime.now)
    channel: str = ""
    reporter: str = ""
    jumps_from_current: int | None = None


class IntelParser:
    """
    Parses chat messages for intel content.

    Uses pattern matching to identify:
    - EVE system names
    - Hostile/threat indicators
    - Ship types
    - Player counts
    """

    # Common EVE ship types for detection (lowercase)
    SHIP_TYPES: set[str] = {
        # Frigates
        "atron",
        "tristan",
        "merlin",
        "rifter",
        "punisher",
        "tormentor",
        "executioner",
        "condor",
        "slasher",
        "incursus",
        "kestrel",
        "breacher",
        "magnate",
        "heron",
        "probe",
        "imicus",
        # Interceptors
        "ares",
        "taranis",
        "crow",
        "raptor",
        "crusader",
        "malediction",
        "claw",
        "stiletto",
        # Assault Frigates
        "enyo",
        "ishkur",
        "harpy",
        "hawk",
        "retribution",
        "vengeance",
        "jaguar",
        "wolf",
        # Interdictors
        "sabre",
        "heretic",
        "flycatcher",
        "eris",
        # Destroyers
        "catalyst",
        "algos",
        "cormorant",
        "corax",
        "coercer",
        "dragoon",
        "thrasher",
        "talwar",
        # Cruisers
        "thorax",
        "vexor",
        "caracal",
        "moa",
        "rupture",
        "stabber",
        "omen",
        "maller",
        "arbitrator",
        "bellicose",
        "celestis",
        "blackbird",
        # Heavy Assault Cruisers
        "deimos",
        "ishtar",
        "cerberus",
        "eagle",
        "vagabond",
        "muninn",
        "zealot",
        "sacrilege",
        # Recon Ships
        "lachesis",
        "arazu",
        "falcon",
        "rook",
        "rapier",
        "huginn",
        "curse",
        "pilgrim",
        # Strategic Cruisers (T3C)
        "proteus",
        "tengu",
        "loki",
        "legion",
        # Battlecruisers
        "brutix",
        "myrmidon",
        "ferox",
        "drake",
        "harbinger",
        "prophecy",
        "hurricane",
        "cyclone",
        "gnosis",
        # Command Ships
        "astarte",
        "eos",
        "vulture",
        "nighthawk",
        "absolution",
        "damnation",
        "sleipnir",
        "claymore",
        # Battleships
        "megathron",
        "hyperion",
        "dominix",
        "raven",
        "rokh",
        "scorpion",
        "typhoon",
        "tempest",
        "maelstrom",
        "apocalypse",
        "armageddon",
        "abaddon",
        "machariel",
        "nightmare",
        "vindicator",
        "rattlesnake",
        "barghest",
        "nestor",
        # Capitals
        "carrier",
        "dreadnought",
        "fax",
        "supercarrier",
        "titan",
        # Specific Capitals
        "nyx",
        "hel",
        "aeon",
        "wyvern",
        "avatar",
        "erebus",
        "ragnarok",
        "leviathan",
        "moros",
        "naglfar",
        "revelation",
        "phoenix",
        "ninazu",
        "lif",
        "apostle",
        "minokawa",
        "thanatos",
        "archon",
        "chimera",
        "nidhoggur",
        # Force Auxiliaries
        "force auxiliary",
        # Other notable ships
        "stratios",
        "astero",
        "svipul",
        "confessor",
        "hecate",
        "jackdaw",
    }

    # Capital ship types (subset for threat escalation)
    CAPITAL_SHIPS: set[str] = {
        "carrier",
        "dreadnought",
        "fax",
        "supercarrier",
        "titan",
        "nyx",
        "hel",
        "aeon",
        "wyvern",
        "avatar",
        "erebus",
        "ragnarok",
        "leviathan",
        "moros",
        "naglfar",
        "revelation",
        "phoenix",
        "ninazu",
        "lif",
        "apostle",
        "minokawa",
        "thanatos",
        "archon",
        "chimera",
        "nidhoggur",
        "force auxiliary",
    }

    # Intel keywords
    HOSTILE_KEYWORDS: set[str] = {
        "hostile",
        "hostiles",
        "neut",
        "neuts",
        "neutral",
        "neutrals",
        "red",
        "reds",
        "spike",
        "spiked",
        "gang",
        "fleet",
        "camp",
        "gate camp",
        "gatecamp",
        "bubble",
        "bubbled",
        "drag",
        "tackled",
        "tackle",
        "point",
        "pointed",
        "warp disrupted",
        "cyno",
        "cyno up",
        "hot",
        "contact",
        "contacts",
    }

    CLEAR_KEYWORDS: set[str] = {
        "clear",
        "clr",
        "empty",
        "nv",
        "no visual",
        "no vis",
        "nothing",
        "safe",
        "all clear",
    }

    MOVEMENT_KEYWORDS: set[str] = {
        "jumping",
        "jumped",
        "holding",
        "warping",
        "landing",
        "inbound",
        "outbound",
        "heading",
        "moving",
    }

    def __init__(
        self,
        known_systems: set[str] | None = None,
        jump_calculator: "JumpCalculator | None" = None,
    ):
        """
        Initialize the intel parser.

        Args:
            known_systems: Optional set of known system names (lowercase)
            jump_calculator: Optional JumpCalculator for distance lookups
        """
        from .systems import load_systems

        self.logger = logging.getLogger(__name__)
        self.known_systems: set[str] = known_systems or load_systems()
        self.jump_calculator = jump_calculator
        self._current_system: str | None = None

    def set_current_system(self, system: str):
        """Set the pilot's current system for jump distance calculation.

        Args:
            system: Current system name (case-insensitive).
        """
        self._current_system = system

    def add_known_system(self, system: str):
        """Add a system to the known systems set."""
        self.known_systems.add(system.lower())

    def parse(
        self,
        message: str,
        timestamp: datetime | None = None,
        channel: str = "",
        reporter: str = "",
    ) -> IntelReport | None:
        """
        Parse a message for intel content.

        Args:
            message: Chat message text
            timestamp: Message timestamp
            channel: Source channel name
            reporter: Player who sent the message

        Returns:
            IntelReport if intel detected, None otherwise
        """
        msg_lower = message.lower()

        # Check for clear reports first
        if any(kw in msg_lower for kw in self.CLEAR_KEYWORDS):
            system = self._extract_system(message)
            if system:
                report = IntelReport(
                    system=system,
                    threat_level=ThreatLevel.CLEAR,
                    hostile_count=0,
                    ship_types=[],
                    player_names=[],
                    raw_message=message,
                    timestamp=timestamp or datetime.now(),
                    channel=channel,
                    reporter=reporter,
                )
                self._populate_jumps(report)
                return report

        # Check for hostile indicators
        has_hostile_keyword = any(kw in msg_lower for kw in self.HOSTILE_KEYWORDS)

        # Extract system name
        system = self._extract_system(message)

        # Extract ship types mentioned
        ships = self._extract_ships(message)

        # Extract count (e.g., "+5", "x3", "5 in local")
        count = self._extract_count(message)

        # Extract player names (basic)
        players = self._extract_players(message)

        # If we have a system and hostile indicators, it's intel
        if system and (has_hostile_keyword or ships or count):
            threat_level = self._assess_threat(count, ships)
            report = IntelReport(
                system=system,
                threat_level=threat_level,
                hostile_count=count,
                ship_types=ships,
                player_names=players,
                raw_message=message,
                timestamp=timestamp or datetime.now(),
                channel=channel,
                reporter=reporter,
            )
            self._populate_jumps(report)
            return report

        # Check if it looks like intel even without a system
        # (e.g., just "hostile Loki +5" in an intel channel)
        if has_hostile_keyword and (ships or count):
            threat_level = self._assess_threat(count, ships)
            return IntelReport(
                system=None,
                threat_level=threat_level,
                hostile_count=count,
                ship_types=ships,
                player_names=players,
                raw_message=message,
                timestamp=timestamp or datetime.now(),
                channel=channel,
                reporter=reporter,
            )

        return None

    def _populate_jumps(self, report: IntelReport):
        """Populate jumps_from_current on a report if possible.

        Requires both a jump_calculator and a current system to be set,
        and the report must have a non-None system.
        """
        if self.jump_calculator and self._current_system and report.system:
            report.jumps_from_current = self.jump_calculator.distance(
                self._current_system, report.system
            )

    def _extract_system(self, message: str) -> str | None:
        """
        Extract system name from message.

        Args:
            message: Chat message

        Returns:
            System name if found, None otherwise
        """
        # Pattern for null-sec system names:
        # - All caps with numbers/dashes: HED-GP, 1DQ1-A, B-R5RB, Y-2ANO
        null_pattern = r"\b([A-Z0-9]{1,4}-[A-Z0-9]{1,6})\b"
        match = re.search(null_pattern, message)
        if match:
            return match.group(1)

        # Also try lowercase version
        null_pattern_lower = r"\b([a-z0-9]{1,4}-[a-z0-9]{1,6})\b"
        match = re.search(null_pattern_lower, message.lower())
        if match:
            return match.group(1).upper()

        # Check against known systems (case-insensitive)
        words = re.findall(r"\b[\w-]+\b", message)
        for word in words:
            if word.lower() in self.known_systems:
                return word

        # Multi-word system names (e.g., "Old Man Star")
        msg_lower = message.lower()
        for system in self.known_systems:
            if " " in system and system in msg_lower:
                return system.title()

        return None

    def _extract_ships(self, message: str) -> list[str]:
        """
        Extract ship types from message.

        Tokenizes the message once and uses set intersection instead of
        running 200+ regex searches per message.

        Args:
            message: Chat message

        Returns:
            List of ship types found
        """
        # Tokenize into lowercase words — handles most single-word ship names
        words = set(re.findall(r"[a-z]+", message.lower()))
        # Single-word ships via fast set intersection
        found = list(words & self.SHIP_TYPES)
        # Multi-word ship names (e.g. "force auxiliary") need substring check
        msg_lower = message.lower()
        for ship in self.SHIP_TYPES:
            if " " in ship and ship in msg_lower:
                found.append(ship)
        return found

    def _extract_count(self, message: str) -> int | None:
        """
        Extract hostile count from message.

        Args:
            message: Chat message

        Returns:
            Count if found, None otherwise
        """
        msg_lower = message.lower()

        # Patterns for count extraction (in order of specificity)
        patterns = [
            r"\+(\d+)",  # +5
            r"x(\d+)",  # x5
            r"(\d+)x\b",  # 5x
            r"(\d+)\s*in\s*local",  # 5 in local
            r"gang\s*of\s*(\d+)",  # gang of 5
            r"fleet\s*of\s*(\d+)",  # fleet of 5
            r"(\d+)\s*hostiles?",  # 5 hostile(s)
            r"(\d+)\s*neuts?",  # 5 neut(s)
            r"(\d+)\s*reds?",  # 5 red(s)
            r"(\d+)\s*man",  # 5 man (gang)
            r"(\d{1,3})\s*spike",  # 50 spike
        ]

        for pattern in patterns:
            match = re.search(pattern, msg_lower)
            if match:
                count = int(match.group(1))
                if 1 <= count <= 500:  # Sanity check
                    return count

        return None

    def _extract_players(self, message: str) -> list[str]:
        """
        Extract player names from message (basic heuristic).

        Args:
            message: Chat message

        Returns:
            List of potential player names
        """
        # This is a basic implementation
        # In production, could cross-reference with ESI character search
        # or maintain a cache of known hostile players

        # Look for patterns like "Player Name" (capitalized words)
        # but filter out ship names, system names, and keywords
        players = []

        # Simple pattern: 2-3 capitalized words together
        name_pattern = r"\b([A-Z][a-z]+(?:\s+[A-Z][a-z]+){1,2})\b"
        matches = re.findall(name_pattern, message)

        for match in matches:
            # Filter out known ship names and keywords
            if match.lower() not in self.SHIP_TYPES:
                if match.lower() not in self.HOSTILE_KEYWORDS:
                    if match.lower() not in self.CLEAR_KEYWORDS:
                        players.append(match)

        return players[:5]  # Limit to 5 names

    def _assess_threat(self, count: int | None, ships: list[str]) -> ThreatLevel:
        """
        Assess threat level based on intel.

        Args:
            count: Number of hostiles
            ships: Ship types detected

        Returns:
            Appropriate ThreatLevel
        """
        # Capital ships = critical threat
        if any(ship in self.CAPITAL_SHIPS for ship in ships):
            return ThreatLevel.CRITICAL

        # Supercarrier/Titan specifically
        super_ships = {
            "supercarrier",
            "titan",
            "nyx",
            "hel",
            "aeon",
            "wyvern",
            "avatar",
            "erebus",
            "ragnarok",
            "leviathan",
        }
        if any(ship in super_ships for ship in ships):
            return ThreatLevel.CRITICAL

        # Large fleet (10+)
        if count and count >= 10:
            return ThreatLevel.DANGER

        # Medium gang (5-9)
        if count and count >= 5:
            return ThreatLevel.WARNING

        # Small gang (2-4) or any hostile report
        if count and count >= 2:
            return ThreatLevel.WARNING

        # Single hostile or ship sighting
        if ships or count:
            return ThreatLevel.INFO

        # Default for any hostile mention
        return ThreatLevel.WARNING

    def is_likely_intel(self, message: str) -> bool:
        """
        Quick check if a message is likely intel (for filtering).

        Args:
            message: Chat message

        Returns:
            True if message appears to be intel
        """
        msg_lower = message.lower()

        # Clear report
        if any(kw in msg_lower for kw in self.CLEAR_KEYWORDS):
            return True

        # Hostile keyword
        if any(kw in msg_lower for kw in self.HOSTILE_KEYWORDS):
            return True

        # Has a system name pattern
        if re.search(r"\b[A-Z0-9]{1,4}-[A-Z0-9]{1,6}\b", message):
            return True

        # Contains a ship type
        if any(ship in msg_lower for ship in self.SHIP_TYPES):
            return True

        # Contains a count pattern
        if re.search(r"\+\d+|x\d+|\d+x\b", msg_lower):
            return True

        return False



"""
UI modules for Argus Overview

v2.4: Added ActionRegistry for UI action management
"""

from argus_overview.ui.action_registry import (
    ActionRegistry,
    ActionScope,
    ActionSpec,
    PrimaryHome,
    audit_actions,
    print_audit_report,
)
from argus_overview.ui.menu_builder import (
    ContextMenuBuilder,
    MenuBuilder,
    ToolbarBuilder,
    build_toolbar_actions,
)

__all__ = [
    "ActionRegistry",
    "ActionScope",
    "ActionSpec",
    "PrimaryHome",
    "ContextMenuBuilder",
    "MenuBuilder",
    "ToolbarBuilder",
    "audit_actions",
    "print_audit_report",
    "build_toolbar_actions",
]



#!/usr/bin/env python3
"""
Argus Overview - Cross-Platform Edition
Main entry point with professional UI and all features

v3.0 Features:
- Cross-platform support (Linux X11, Windows Win32)
- Platform abstraction layer
- Unified codebase for all platforms

v2.4 Features:
- ActionRegistry: Single source of truth for all UI actions
- Centralized menu/toolbar building from registry
- Tab renames: Overview, Cycle Control, Roster, Sync
- ToolbarBuilder and ContextMenuBuilder classes
- CLI audit tool for redundancy checking

v2.2 Features:
- System tray with minimize-to-tray
- One-click EVE window import
- Auto-discovery of new EVE clients
- Per-character hotkeys
- Thumbnail hover effects (opacity/zoom)
- Activity indicators
- Session timers
- Custom labels for characters
- Themes (Dark, Light, EVE)
- Hot reload configuration
- Position lock for thumbnails
- Single instance enforcement
"""

import logging
import os
import sys
from io import TextIOWrapper
from pathlib import Path

# Platform-specific imports for single-instance locking
if sys.platform == "win32":
    import msvcrt
else:
    import fcntl

from PySide6.QtCore import Qt
from PySide6.QtGui import QColor, QPalette
from PySide6.QtWidgets import QApplication, QMessageBox

from argus_overview import __version__

# Add src to path
sys.path.insert(0, str(Path(__file__).parent))

# NOTE: MainWindowV21 is imported INSIDE main() AFTER single-instance check
# to prevent race conditions from early imports starting background threads


class SingleInstance:
    """
    Ensures only one instance of the application can run at a time.
    Uses fcntl on Linux/macOS and msvcrt on Windows for reliable locking.
    """

    def __init__(self, app_name: str = "argus-overview"):
        self.app_name = app_name
        self.lock_file: TextIOWrapper | None = None

        # Platform-specific lock file location
        if sys.platform == "win32":
            # Windows: Use AppData/Local
            app_data = os.environ.get("LOCALAPPDATA", Path.home() / "AppData" / "Local")
            self.lock_path = Path(app_data) / "argus-overview" / f"{app_name}.lock"
        else:
            # Linux/macOS: Use ~/.config
            self.lock_path = Path.home() / ".config" / "argus-overview" / f"{app_name}.lock"

        # Ensure directory exists
        self.lock_path.parent.mkdir(parents=True, exist_ok=True)

    def try_lock(self) -> bool:
        """
        Try to acquire the lock.
        Returns True if successful (first instance), False if already running.
        """
        try:
            self.lock_file = open(self.lock_path, "w")
            try:
                if sys.platform == "win32":
                    # Windows: Use msvcrt.locking
                    msvcrt.locking(self.lock_file.fileno(), msvcrt.LK_NBLCK, 1)
                else:
                    # Linux/macOS: Use fcntl.flock
                    fcntl.flock(self.lock_file.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
                # Write PID to lock file
                self.lock_file.write(str(os.getpid()))
                self.lock_file.flush()
                return True
            except Exception:
                # Lock failed or other error - clean up file handle
                self.lock_file.close()
                self.lock_file = None
                raise
        except OSError:
            # Lock is held by another instance or file operation failed
            return False

    def release(self):
        """Release the lock"""
        if self.lock_file:
            try:
                if sys.platform == "win32":
                    # Windows: Unlock with msvcrt
                    msvcrt.locking(self.lock_file.fileno(), msvcrt.LK_UNLCK, 1)
                else:
                    # Linux/macOS: Unlock with fcntl
                    fcntl.flock(self.lock_file.fileno(), fcntl.LOCK_UN)
                self.lock_file.close()
            except (OSError, ValueError):
                pass  # File already closed or released, safe to ignore
            self.lock_file = None

    def __enter__(self):
        return self.try_lock()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.release()


def setup_logging():
    """Setup logging configuration"""
    log_dir = Path.home() / ".config" / "argus-overview"
    log_dir.mkdir(parents=True, exist_ok=True)

    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        handlers=[logging.StreamHandler(), logging.FileHandler(log_dir / "argus-overview.log")],
    )


def setup_dark_theme(app):
    """Setup professional dark theme"""
    app.setStyle("Fusion")

    palette = QPalette()
    palette.setColor(QPalette.ColorRole.Window, QColor(53, 53, 53))
    palette.setColor(QPalette.ColorRole.WindowText, Qt.GlobalColor.white)
    palette.setColor(QPalette.ColorRole.Base, QColor(25, 25, 25))
    palette.setColor(QPalette.ColorRole.AlternateBase, QColor(53, 53, 53))
    palette.setColor(QPalette.ColorRole.ToolTipBase, Qt.GlobalColor.white)
    palette.setColor(QPalette.ColorRole.ToolTipText, Qt.GlobalColor.white)
    palette.setColor(QPalette.ColorRole.Text, Qt.GlobalColor.white)
    palette.setColor(QPalette.ColorRole.Button, QColor(53, 53, 53))
    palette.setColor(QPalette.ColorRole.ButtonText, Qt.GlobalColor.white)
    palette.setColor(QPalette.ColorRole.BrightText, Qt.GlobalColor.red)
    palette.setColor(QPalette.ColorRole.Link, QColor(42, 130, 218))
    palette.setColor(QPalette.ColorRole.Highlight, QColor(42, 130, 218))
    palette.setColor(QPalette.ColorRole.HighlightedText, Qt.GlobalColor.black)
    app.setPalette(palette)


def check_wayland_compatibility() -> bool:
    """Check for Wayland compatibility and warn user if needed.

    Returns:
        True if should continue, False if user cancelled.
    """
    from argus_overview.utils.display_server import (
        DisplayServer,
        detect_display_server,
        get_wayland_limitation_message,
    )

    info = detect_display_server()

    if info.server == DisplayServer.WAYLAND and not info.has_x11_access:
        # Pure Wayland - show warning
        msg_box = QMessageBox()
        msg_box.setIcon(QMessageBox.Icon.Warning)
        msg_box.setWindowTitle("Wayland Detected")
        msg_box.setText("Pure Wayland session detected - limited functionality")
        msg_box.setInformativeText(get_wayland_limitation_message())
        msg_box.setStandardButtons(
            QMessageBox.StandardButton.Ok | QMessageBox.StandardButton.Cancel
        )
        msg_box.setDefaultButton(QMessageBox.StandardButton.Cancel)

        result = msg_box.exec()
        if result == QMessageBox.StandardButton.Cancel:
            return False

    elif info.server == DisplayServer.XWAYLAND:
        # XWayland - log info, should work
        logging.getLogger(__name__).info(
            "Running under XWayland - X11 tools available, full functionality expected"
        )

    return True


def main():
    """Main application entry point"""
    # Setup logging
    setup_logging()

    logger = logging.getLogger(__name__)
    logger.info(f"Starting Argus Overview v{__version__}")

    # Create QApplication early (needed for dialogs)
    app = QApplication(sys.argv)

    # Single instance check
    single_instance = SingleInstance()
    if not single_instance.try_lock():
        logger.warning("Another instance is already running")
        QMessageBox.warning(
            None,
            "Already Running",
            "Argus Overview is already running.\n\n"
            "Check your system tray for the existing instance.",
        )
        sys.exit(1)

    # Check Wayland compatibility before importing X11-dependent modules (Linux only)
    if sys.platform != "win32" and not check_wayland_compatibility():
        logger.info("User cancelled due to Wayland limitations")
        single_instance.release()
        sys.exit(0)

    # Import main window AFTER lock acquired and Wayland check passed
    # (importing can start background threads/X11 hooks)
    from argus_overview.ui.main_window_v21 import MainWindowV21

    # Configure application (QApplication already created above)
    app.setApplicationName("Argus Overview")
    app.setOrganizationName("Argus Overview")
    app.setDesktopFileName("argus-overview")  # Matches .desktop file name

    # Setup theme
    setup_dark_theme(app)

    # Create and show main window
    window = MainWindowV21()
    window.show()

    # Run application
    exit_code = app.exec()

    # Release lock on exit
    single_instance.release()

    sys.exit(exit_code)


if __name__ == "__main__":
    main()
